import { useState } from 'react';
import { AutoComplete, Empty, Input, Space, Tag } from 'antd';
import type { SelectProps } from 'antd';
import { searchQuery } from '@/services/admin';
import styles from './style.less';
import { PAYMENT_STATUS_MAP } from '@/pages/order/payment/constants';
import { history } from 'umi';
import { ROUTER_ENUM } from '@/constants';
import { debounce } from 'lodash';

// type IType = 'order' | 'ip';
type IResultType =
  | 'paymentOrder'
  | 'transferOrder'
  | 'mchLoginLog'
  | 'proxyLoginLog'
  | 'adminLoginLog';
interface IItem extends API.PaymentOrderItem {
  resultType: IResultType;
  [key: string]: any;
}

// const { Option } = Select;
const ORDER_MAP = {
  paymentOrder: '支付订单',
  transferOrder: '代付订单',
};

const LOGIN_MAP = {
  mchLoginLog: '商户',
  proxyLoginLog: '代理',
  adminLoginLog: '管理员',
};

// const TYPE_MAP = {
//   order: '订单号',
//   ip: 'IP',
// };

const shouldHighlight = (v1: string, v2: string) => v1 === v2;

function HeaderSearch() {
  // const [type, setType] = useState<IType>('order');
  const [searchValue, setSearchValue] = useState<string>('');
  const [options, setOptions] = useState<SelectProps<Record<string, unknown>>['options']>([]);

  const resetData = () => {
    // setType('order');
    setSearchValue('');
    setOptions([]);
  };

  // 跳具体页面
  const toDetailPage = (item: IItem) => {
    resetData();
    if (item.resultType === 'paymentOrder') {
      history.replace({
        pathname: ROUTER_ENUM.orderPayment,
        query: {
          mchOrderNo: item.mchOrderNo,
        },
      });
    } else if (item.resultType === 'transferOrder') {
      history.replace({
        pathname: ROUTER_ENUM.orderTransfer,
        query: {
          mchOrderNo: item.mchOrderNo,
        },
      });
    } else if (item.resultType === 'mchLoginLog') {
      history.replace({
        pathname: ROUTER_ENUM.loginMerchant,
        query: {
          loginIp: item.ip as string,
        },
      });
    } else if (item.resultType === 'proxyLoginLog') {
      history.replace({
        pathname: ROUTER_ENUM.loginProxy,
        query: {
          loginIp: item.ip as string,
        },
      });
    } else if (item.resultType === 'adminLoginLog') {
      history.replace({
        pathname: ROUTER_ENUM.loginAdmin,
        query: {
          loginIp: item.ip as string,
        },
      });
    }
  };

  // 生成相应的option
  const getOption = (searchVal: string, item: IItem) => {
    if (['transferOrder', 'paymentOrder'].includes(item.resultType)) {
      return {
        value: item.mchId,
        label: (
          <div className={styles.optionContainer} onClick={toDetailPage.bind(null, item)}>
            <div className={styles.optionLeft}>
              <div className={styles.optionsTitle}>{ORDER_MAP[item.resultType]}</div>
              <div className={styles.optionLeftItem}>
                <Tag color={PAYMENT_STATUS_MAP[item.statusCode].status}>
                  {PAYMENT_STATUS_MAP[item.statusCode].text}
                </Tag>
              </div>
              <div>
                <strong className="orange">{item.amount}</strong>
              </div>
            </div>
            <div className={styles.optionRight}>
              <Space direction="vertical">
                <span>
                  商户交易号：
                  <span className={shouldHighlight(searchVal, item.mchOrderNo) ? 'highlight' : ''}>
                    {item.mchOrderNo}
                  </span>
                </span>
                <span>
                  平台流水号：
                  <span
                    className={shouldHighlight(searchVal, item.serialOrderNo) ? 'highlight' : ''}
                  >
                    {item.serialOrderNo}
                  </span>
                </span>
                <span>
                  通道订单号：
                  <span
                    className={shouldHighlight(searchVal, item.channelOrderNo) ? 'highlight' : ''}
                  >
                    {item.channelOrderNo}
                  </span>
                </span>
                <span>
                  通道名称：{item.channelName}
                  {item.ip && (
                    <span style={{ float: 'right' }}>
                      IP：
                      <span
                        className={shouldHighlight(searchVal, item.ip as string) ? 'highlight' : ''}
                      >
                        {item.ip}
                      </span>
                    </span>
                  )}
                </span>
              </Space>
            </div>
          </div>
        ),
      };
    } else if (['mchLoginLog', 'proxyLoginLog', 'adminLoginLog'].includes(item.resultType)) {
      return {
        value: item.mchId,
        label: (
          <div className={styles.optionContainer} onClick={toDetailPage.bind(null, item)}>
            <div className={styles.optionLeft}>
              <div className={styles.optionsTitle}>{LOGIN_MAP[item.resultType]}登录</div>
              <div>
                {item.loginResult ? (
                  <Tag color="success">登录成功</Tag>
                ) : (
                  <Tag color="error">登录失败</Tag>
                )}
              </div>
            </div>
            <div className={styles.optionRight}>
              <Space direction="vertical">
                <span>
                  {LOGIN_MAP[item.resultType]}名称：{item.domainName}
                </span>
                <span>登录账号：{item.loginName}</span>
                <span>登录时间：{item.loginTime}</span>
                <span>
                  登录IP：
                  <span
                    className={shouldHighlight(searchVal, item.ip as string) ? 'highlight' : ''}
                  >
                    {item.ip}
                  </span>
                </span>
              </Space>
            </div>
          </div>
        ),
      };
    }
    return null;
  };
  const handleSearch = async (value: string) => {
    setSearchValue(value);
    if (!value) {
      setOptions([]);
      return;
    }

    const res = await searchQuery({ key: value });
    if (res?.code === '0000') {
      let curOptions = res.data.map(getOption.bind(null, value));
      if (!curOptions.length) {
        curOptions = [{ value: 'blank', label: <Empty className={styles.empty} /> }];
      }
      setOptions(curOptions);
    }
  };

  // const handleSelect = (value: IType) => {
  //   setType(value);
  //   if (searchValue) {
  //     handleSearch(searchValue, value);
  //   }
  // };

  const handleFocus = () => {
    if (searchValue) {
      handleSearch(searchValue);
    }
  };

  return (
    <div className={styles.container}>
      {/* <Select
        className={styles.searchSelect}
        value={type}
        style={{ width: 80 }}
        onSelect={handleSelect}
      >
        <Option value="order">{TYPE_MAP.order}</Option>
        <Option value="ip">{TYPE_MAP.ip}</Option>
      </Select> */}
      <AutoComplete
        allowClear
        className={styles.searchInput}
        dropdownClassName={styles.searchDropdown}
        dropdownMatchSelectWidth={500}
        style={{ width: 500 }}
        options={options}
        defaultValue={searchValue}
        onSearch={debounce(handleSearch, 250)}
        onFocus={handleFocus}
      >
        <Input.Search placeholder={`请输入订单号/IP地址进行搜索`} enterButton />
      </AutoComplete>
    </div>
  );
}

export default HeaderSearch;
