import React, { useCallback } from 'react';
import { GoogleOutlined, LogoutOutlined, SettingOutlined, UserOutlined } from '@ant-design/icons';
import { Avatar, Menu, Spin, Tag } from 'antd';
import { history, useModel } from 'umi';
import { stringify } from 'querystring';
import HeaderDropdown from '../HeaderDropdown';
import styles from './index.less';
import { outLogin, requestResetPwd } from '@/services/admin';
import type { MenuInfo } from 'rc-menu/lib/interface';
import { ModalForm, ProFormText } from '@ant-design/pro-form';
import { passwordValidate } from '@/utils/validators';

export type GlobalHeaderRightProps = {
  menu?: boolean;
};

/**
 * 退出登录，并且将当前的 url 保存
 */
const loginOut = async () => {
  await outLogin();
  const { query = {}, pathname } = history.location;
  const { redirect } = query;
  // Note: There may be security issues, please note
  if (window.location.pathname !== '/admin/login' && !redirect) {
    history.replace({
      pathname: '/admin/login',
      search: stringify({
        redirect: pathname,
      }),
    });
  }
};

const AvatarDropdown: React.FC<GlobalHeaderRightProps> = ({ menu }) => {
  const { initialState, setInitialState } = useModel('@@initialState');

  const onMenuClick = useCallback(
    (event: MenuInfo) => {
      const { key } = event;
      console.log('onMenuClick: ', key);

      if (key === 'logout') {
        setInitialState((s) => ({ ...s, currentUser: undefined }));
        loginOut();
        return;
      }
      if (key === 'googleCertify' && !initialState?.currentUser?.googleStatus) {
        history.push(`/admin/verify`);
        return;
      }
      // history.push(`/account/${key}`);
    },
    [setInitialState, initialState],
  );

  const loading = (
    <span className={`${styles.action} ${styles.account}`}>
      <Spin
        size="small"
        style={{
          marginLeft: 8,
          marginRight: 8,
        }}
      />
    </span>
  );

  if (!initialState) {
    return loading;
  }

  const { currentUser } = initialState;

  if (!currentUser || !currentUser.adminName) {
    return loading;
  }

  const menuHeaderDropdown = (
    <Menu className={styles.menu} selectedKeys={[]} onClick={onMenuClick}>
      {/* {menu && (
        <Menu.Item key="center">
          <UserOutlined />
          个人中心
        </Menu.Item>
      )}
       */}
      {menu && (
        <Menu.Item key="googleCertify">
          <GoogleOutlined />
          谷歌验证&nbsp;&nbsp;
          {currentUser.googleStatus ? (
            <Tag color="success">已验证</Tag>
          ) : (
            <Tag color="warning">未验证</Tag>
          )}
        </Menu.Item>
      )}
      {menu && <Menu.Divider />}
      {menu && (
        <ModalForm
          title="修改密码"
          width={480}
          onFinish={requestResetPwd}
          trigger={
            <Menu.Item key="settings">
              <SettingOutlined />
              修改密码
            </Menu.Item>
          }
        >
          <ProFormText.Password
            name="oldPwd"
            label="旧密码"
            width={430}
            placeholder="请输入旧密码"
            rules={[
              {
                required: true,
                validator: passwordValidate,
              },
            ]}
          />
          <ProFormText.Password
            width={430}
            name="newPwd"
            label="新密码"
            placeholder="请输入新密码"
            rules={[
              {
                required: true,
                validator: passwordValidate,
              },
            ]}
          />
          <ProFormText.Password
            width={430}
            name="newPwd2"
            label="确认新密码"
            placeholder="请再次输入新密码"
            rules={[
              ({ getFieldValue }) => ({
                required: true,
                validator(_, value) {
                  if (!value) {
                    return Promise.reject(new Error('请输入密码!'));
                  }
                  if (value.length < 6 || value.length > 20) {
                    return Promise.reject(new Error('6-20位字符'));
                  }
                  if (!value || getFieldValue('newPwd') === value) {
                    return Promise.resolve();
                  }
                  return Promise.reject(new Error('两次输入密码不一致!'));
                },
              }),
            ]}
          />
        </ModalForm>
      )}
      {menu && <Menu.Divider />}
      <Menu.Item key="logout">
        <LogoutOutlined />
        退出登录
      </Menu.Item>
    </Menu>
  );
  return (
    <HeaderDropdown overlay={menuHeaderDropdown}>
      <span className={`${styles.action} ${styles.account}`}>
        <Avatar size="small" className={styles.avatar} icon={<UserOutlined />} alt="avatar" />
        <span className={`${styles.name} anticon`}>{currentUser.adminName}</span>
      </span>
    </HeaderDropdown>
  );
};

export default AvatarDropdown;
