import { useRef } from 'react';
import { message } from 'antd';
import {
  DrawerForm,
  ProFormDependency,
  ProFormDigit,
  ProFormMoney,
  ProFormRadio,
  ProFormSelect,
} from '@ant-design/pro-form';
import type { ProFormInstance } from '@ant-design/pro-form';
import { requestMerchantOptions } from '@/services/commons';
import { getPaymentChannelConfig, savePaymentChannel } from '@/services/merchant/channelPayment';
import { requestChannelOptions, requestProductOptions } from '@/services/merchant/payment';

const formItemLayout = {
  labelCol: {
    style: {
      width: '104px',
    },
  },
  wrapperCol: { span: 18 },
};
const showFeeFix = (feeType: number) => [2, 3].includes(feeType);
const showFeeRate = (feeType: number) => [1, 3].includes(feeType);

interface IProps {
  initialData: API.MerchantChannelItem;
  refresh?: () => void | undefined;
}

function ModalFormEdit({ initialData, refresh }: IProps) {
  const formRef = useRef<ProFormInstance>();

  const handleFinish = async (values: API.MerchantChannelItem) => {
    const res = await savePaymentChannel(values);
    if (res.code === '0000') {
      refresh?.();
      formRef.current?.resetFields();
      message.success('保存成功');
      return true;
    }
    return false;
  };

  const handleSelect = async () => {
    const { mchId, productId, channelId } = formRef.current?.getFieldsValue();
    if (mchId && productId && channelId) {
      const res = await getPaymentChannelConfig({
        mchId,
        productId,
        channelId,
      });
      console.log(res, 'res....');
      if (res && res?.code === '0000') {
        const { feeType, feeFix, feeRate, weight, dayLimitAmountValue } = res.data;
        formRef.current?.setFieldsValue({
          feeType,
          feeFix,
          feeRate,
          weight,
          dayLimitAmountValue,
        });
      }
    }
  };

  return (
    <div>
      <DrawerForm
        {...formItemLayout}
        formRef={formRef}
        title="通道分流"
        layout="horizontal"
        width={660}
        trigger={<a>分流</a>}
        onFinish={handleFinish}
        initialValues={initialData}
      >
        <ProFormSelect
          showSearch
          name="mchId"
          placeholder="请选择商户"
          label="商户"
          request={requestMerchantOptions}
          fieldProps={{ onSelect: handleSelect }}
          rules={[{ required: true, message: '请选择商户!' }]}
        />
        <ProFormSelect
          showSearch
          name="productId"
          placeholder="请选择大通道"
          label="大通道"
          request={requestProductOptions}
          fieldProps={{ onSelect: handleSelect }}
          rules={[{ required: true, message: '请选择大通道!' }]}
        />
        <ProFormSelect
          showSearch
          name="channelId"
          placeholder="请选择子通道"
          label="子通道"
          request={requestChannelOptions}
          fieldProps={{ onSelect: handleSelect }}
          rules={[{ required: true, message: '请选择子通道!' }]}
        />
        <ProFormRadio.Group
          name="feeType"
          label="收费方式"
          options={[
            {
              label: '指定费率',
              value: 1,
            },
            {
              label: '单笔固额',
              value: 2,
            },
            {
              label: '组合收费',
              value: 3,
            },
          ]}
          rules={[{ required: true, message: '请选择收费方式!' }]}
        />
        <ProFormDependency name={['feeType']}>
          {({ feeType }) => {
            return (
              <>
                {showFeeRate(feeType) && (
                  <ProFormDigit
                    name="feeRate"
                    min={1}
                    max={100}
                    fieldProps={{ addonAfter: '%' }}
                    placeholder="请输入通道费率"
                    label="通道费率"
                    required
                    rules={[
                      ({ getFieldValue }) => ({
                        validator(_, value) {
                          const feeTypeValue = getFieldValue('feeType');
                          if (!value && showFeeRate(feeTypeValue)) {
                            return Promise.reject('请输入通道费率!');
                          }
                          return Promise.resolve();
                        },
                      }),
                    ]}
                  />
                )}
                {showFeeFix(feeType) && (
                  <ProFormDigit
                    name="feeFix"
                    min={1}
                    max={20000000}
                    fieldProps={{ addonAfter: '元' }}
                    placeholder="请输入单笔费用"
                    label="单笔费用"
                    required
                    rules={[
                      ({ getFieldValue }) => ({
                        validator(_, value) {
                          const feeTypeValue = getFieldValue('feeType');
                          if (!value && showFeeFix(feeTypeValue)) {
                            return Promise.reject('请输入单笔费用!');
                          }
                          return Promise.resolve();
                        },
                      }),
                    ]}
                  />
                )}
              </>
            );
          }}
        </ProFormDependency>
        <ProFormMoney
          name="dayLimitAmountValue"
          min={0}
          max={20000000}
          placeholder="请输入日总限额"
          label="日总限额"
          customSymbol={initialData.currencyMark || '¥'}
          rules={[{ required: true, message: '请输入日总限额!' }]}
        />
        <ProFormDigit
          name="weight"
          min={1}
          max={10}
          placeholder="请输入权重"
          label="权重"
          rules={[{ required: true, message: '请输入权重!' }]}
        />
      </DrawerForm>
    </div>
  );
}

export default ModalFormEdit;
