import { useState, useRef } from 'react';
import {
  ModalForm,
  ProFormDependency,
  ProFormDigit,
  ProFormMoney,
  ProFormRadio,
  ProFormSwitch,
} from '@ant-design/pro-form';
import type { ProFormInstance } from '@ant-design/pro-form';
import { Alert, message, Radio } from 'antd';
import { updatePaymentBatch } from '@/services/merchant/channelPayment';
import styles from '../../style.less';

interface IProps {
  initialData: Partial<API.MerchantChannelItem>;
  refresh?: any;
  children?: JSX.Element;
}

const formItemLayout = {
  labelCol: {
    style: {
      width: '104px',
    },
  },
  wrapperCol: {
    sm: { span: 16 },
  },
};

type IType = 'feeRate' | 'status' | 'weight' | 'dayLimitAmountValue';
const TYPE_ENUMS: Record<string, IType> = {
  FEE_RATE: 'feeRate',
  STATUS: 'status',
  WEIGHT: 'weight',
  LIMIT: 'dayLimitAmountValue',
};

const options = [
  { label: '费率', value: TYPE_ENUMS.FEE_RATE },
  { label: '状态', value: TYPE_ENUMS.STATUS },
  { label: '权重', value: TYPE_ENUMS.WEIGHT },
  { label: '日总限额', value: TYPE_ENUMS.LIMIT },
];
const typeMap = {
  [TYPE_ENUMS.FEE_RATE]: '费率',
  [TYPE_ENUMS.STATUS]: '状态',
  [TYPE_ENUMS.WEIGHT]: '权重',
  [TYPE_ENUMS.LIMIT]: '日总限额',
};

const showFeeFix = (feeType: number) => [2, 3].includes(feeType);
const showFeeRate = (feeType: number) => [1, 3].includes(feeType);

function ModalFormFeeBatch({ initialData, refresh, children }: IProps) {
  // 批量编辑类型
  const [type, setType] = useState<IType>(TYPE_ENUMS.FEE_RATE);

  const formRef = useRef<ProFormInstance>();

  const reset = () => {
    setType(TYPE_ENUMS.FEE_RATE);
    formRef.current?.resetFields();
  };

  const handleFinish = async (values: API.MerchantChannelItem) => {
    let params: any = { mchId: initialData.mchId, productId: initialData.productId };
    if (type === TYPE_ENUMS.FEE_RATE) {
      const { feeType, feeRate, feeFix } = values;
      params = { ...params, feeType, feeRate, feeFix };
    } else if (type === TYPE_ENUMS.STATUS) {
      const { status } = values;
      params = { ...params, status: status ? 1 : 0 };
    } else if (type === TYPE_ENUMS.WEIGHT) {
      const { weight } = values;
      params = { ...params, weight };
    } else if (type === TYPE_ENUMS.LIMIT) {
      const { dayLimitAmountValue } = values;
      params = { ...params, dayLimitAmountValue };
    }

    const res = await updatePaymentBatch(params);
    if (res.code === '0000') {
      refresh?.();
      message.success(`批量编辑成功`);
      setTimeout(reset, 1000);
      return true;
    }
    return false;
  };

  return (
    <div>
      <ModalForm
        {...formItemLayout}
        title="批量更新"
        layout="horizontal"
        width={520}
        className={styles.editBatch}
        trigger={children}
        onFinish={handleFinish}
        formRef={formRef}
      >
        <Alert
          message={
            <div>
              <div style={{ marginBottom: 5 }}>
                1. 批量设置商户『{initialData.mchName}』的『{initialData.productName}』
                {typeMap[type]}。
              </div>
              <div>2. 确认后将更新所有子通道的{typeMap[type]}。</div>
            </div>
          }
          type="error"
        />
        <div className={styles.modalContent}>
          <Radio.Group
            style={{ marginBottom: 20 }}
            options={options}
            onChange={({ target: { value } }) => setType(value)}
            value={type}
            optionType="button"
          />

          {type === TYPE_ENUMS.FEE_RATE && (
            <>
              <ProFormRadio.Group
                name="feeType"
                label="收费方式"
                options={[
                  {
                    label: '指定费率',
                    value: 1,
                  },
                  {
                    label: '单笔固额',
                    value: 2,
                  },
                  {
                    label: '组合收费',
                    value: 3,
                  },
                ]}
                rules={[{ required: true, message: '请选择收费方式!' }]}
              />
              <ProFormDependency name={['feeType']}>
                {({ feeType }) => {
                  return (
                    <>
                      {showFeeRate(feeType) && (
                        <ProFormDigit
                          name="feeRate"
                          min={1}
                          max={100}
                          fieldProps={{ addonAfter: '%' }}
                          placeholder="请输入通道费率"
                          label="通道费率"
                          required
                          rules={[
                            ({ getFieldValue }) => ({
                              validator(_, value) {
                                const feeTypeValue = getFieldValue('feeType');
                                if (!value && showFeeRate(feeTypeValue)) {
                                  return Promise.reject('请输入通道费率!');
                                }
                                return Promise.resolve();
                              },
                            }),
                          ]}
                        />
                      )}
                      {showFeeFix(feeType) && (
                        <ProFormDigit
                          name="feeFix"
                          min={1}
                          max={20000000}
                          fieldProps={{ addonAfter: '元' }}
                          placeholder="请输入单笔费用"
                          label="单笔费用"
                          required
                          rules={[
                            ({ getFieldValue }) => ({
                              validator(_, value) {
                                const feeTypeValue = getFieldValue('feeType');
                                if (!value && showFeeFix(feeTypeValue)) {
                                  return Promise.reject('请输入单笔费用!');
                                }
                                return Promise.resolve();
                              },
                            }),
                          ]}
                        />
                      )}
                    </>
                  );
                }}
              </ProFormDependency>
            </>
          )}
          {type === TYPE_ENUMS.STATUS && (
            <ProFormSwitch
              checkedChildren="启用"
              unCheckedChildren="禁用"
              name="status"
              label="状态"
            />
          )}
          {type === TYPE_ENUMS.WEIGHT && (
            <>
              <ProFormDigit
                width={250}
                name="weight"
                min={1}
                max={10}
                placeholder="请输入权重"
                label="权重"
                rules={[{ required: true, message: '请输入权重!' }]}
              />
              <div className="error" style={{ paddingLeft: 104 }}>
                权重值1~10，值越大概率越大
              </div>
            </>
          )}
          {type === TYPE_ENUMS.LIMIT && (
            <ProFormMoney
              width={250}
              name="dayLimitAmountValue"
              min={0}
              max={20000000}
              placeholder="请输入日总限额"
              label="日总限额"
              customSymbol={initialData.currencyMark || '¥'}
              rules={[{ required: true, message: '请输入日总限额!' }]}
            />
          )}
        </div>
      </ModalForm>
    </div>
  );
}

export default ModalFormFeeBatch;
