import { PRODUCT_ICON_ENUM, PROTABLE_DEFAULT_SETTING } from '@/constants';
import ProTable from '@ant-design/pro-table';
import type { ActionType } from '@ant-design/pro-table';
import type { ProColumns } from '@ant-design/pro-table';
import { Avatar, message, Popconfirm, Progress, Space, Switch } from 'antd';
import { useRef, useEffect } from 'react';
import {
  deletePaymentChannel,
  requestPaymentChannelList,
  switchPaymentChannelStatus,
} from '@/services/merchant/channelPayment';
import { requestMerchantOptions } from '@/services/commons';
import ModalFormWeight from './ModalFormWeight';
import ModalFormDayLimitAmount from './ModalFormDayLimitAmount';
import ModalFormFee from './ModalFormFee';
import ModalFormEdit from './ModalFormEdit';
import type { ProFormInstance } from '@ant-design/pro-form';
import { requestChannelOptions, requestProductOptions } from '@/services/merchant/payment';
import ModalFormEditBatch from './ModalFormEditBatch';
import styles from '../../style.less';

function ChannelPayment({ initialData }: { initialData: Partial<API.MerchantChannelItem> }) {
  const actionRef = useRef<ActionType>();
  const formRef = useRef<ProFormInstance>();

  const handleDelete = async (id: string) => {
    const res = await deletePaymentChannel(id);
    if (res?.code === '0000') {
      actionRef.current?.reload();
      message.success('删除成功');
    }
  };

  // 切换状态
  const switchStatus = async (id: string, status: number) => {
    const text = status === 0 ? '禁用' : '启用';
    const res = await switchPaymentChannelStatus(id, status);
    if (res?.code === '0000') {
      actionRef.current?.reload?.();
      message.success(`${text}成功`);
    }
  };

  const columns: ProColumns<API.MerchantChannelItem>[] = [
    {
      title: '商户ID',
      search: false,
      width: 65,
      dataIndex: 'productName',
      render: (text, row) => {
        // const randomColor =
        //   PRODUCT_COLOR_MAP[row.productCode] ||
        //   `bg-color-${(Math.random() * 9 + 1).toFixed(0)}`;
        return {
          children: <div>{row.mchId}</div>,
          props: {
            rowSpan: row.productRowSpan,
            className: `${styles.largeChannel} ${row.bgColor}`,
          },
        };
      },
    },
    {
      title: '商户名称',
      search: false,
      width: 100,
      dataIndex: 'mchName',
      render: (text, row) => {
        // const randomColor =
        //   PRODUCT_COLOR_MAP[row.productCode] ||
        //   `bg-color-${(Math.random() * 9 + 1).toFixed(0)}`;
        return {
          children: <div>{row.mchName}</div>,
          props: {
            rowSpan: row.productRowSpan,
            className: `${styles.largeChannel} ${row.bgColor}`,
          },
        };
      },
    },
    {
      title: '大通道名称',
      search: false,
      width: 130,
      dataIndex: 'productName',
      render: (text, record) => {
        return {
          children: (
            <ModalFormEditBatch initialData={record} refresh={actionRef.current?.reload}>
              <a>{record.productName}</a>
            </ModalFormEditBatch>
          ),
          props: {
            rowSpan: record.productRowSpan,
            className: `${styles.largeChannel} ${record.bgColor}`,
          },
        };
      },
    },
    {
      title: '子通道名称',
      search: false,
      width: 200,
      dataIndex: 'channelName',
      render: (payMethod, record) => (
        <Space>
          <Avatar src={PRODUCT_ICON_ENUM[record.payMethod]} shape="square" size="small" />
          <span>{record.channelName}</span>
        </Space>
      ),
    },
    {
      title: '日总限额',
      search: false,
      width: 145,
      dataIndex: 'dayLimitAmount',
      render: (_, record) => {
        return (
          <Space>
            {_}
            <ModalFormDayLimitAmount initialData={record} refresh={actionRef.current?.reload} />
          </Space>
        );
      },
    },
    {
      title: '今日成功总计',
      search: false,
      width: 110,
      dataIndex: 'todayTotalAmount',
      render: (_) => <strong className="orange">{_}</strong>,
    },
    {
      title: '今日已使用',
      search: false,
      width: 100,
      dataIndex: 'todayRate',
      render: (_, record) => (
        <Progress percent={record.todayRate} steps={10} size="small" strokeColor="#52c41a" />
      ),
    },

    // 搜索项
    {
      title: '商户',
      order: 10,
      dataIndex: 'mchId',
      // initialValue: initialData.mchId ? Number(initialData.mchId) : '',
      hideInTable: true,
      valueType: 'select',
      request: requestMerchantOptions,
      fieldProps: {
        showSearch: true,
      },
    },
    {
      title: '大通道',
      order: 9,
      key: 'productIdPayment',
      dataIndex: 'productIdPayment',
      hideInTable: true,
      valueType: 'select',
      request: requestProductOptions,
      fieldProps: {
        showSearch: true,
      },
    },
    {
      title: '子通道',
      order: 9,
      key: 'channelIdPayment',
      dataIndex: 'channelIdPayment',
      hideInTable: true,
      valueType: 'select',
      request: requestChannelOptions,
      fieldProps: {
        showSearch: true,
      },
    },
    // 搜索项
    {
      title: '权重',
      width: 65,
      search: false,
      dataIndex: 'weight',
      render: (_, record) => {
        return (
          <Space>
            {_}
            <ModalFormWeight initialData={record} refresh={actionRef.current?.reload} />
          </Space>
        );
      },
    },
    {
      title: '费率',
      width: 160,
      search: false,
      dataIndex: 'feeConfig',
      render: (_, record) => {
        return (
          <Space>
            {/* {record.feeConfig.includes('+') ? (
              <Text style={{ width: 70 }} ellipsis={{ tooltip: _ }}>
                {_}
              </Text>
            ) : (
              _
            )} */}
            {_}
            <ModalFormFee initialData={record} refresh={actionRef.current?.reload} />
          </Space>
        );
      },
    },
    {
      title: '状态',
      dataIndex: 'status',
      width: 80,
      render: (_, record) => {
        const { status } = record;
        return (
          <Switch
            // disabled={!googleStatus}
            checkedChildren="启用"
            unCheckedChildren="禁用"
            checked={!!status}
            onClick={switchStatus.bind(null, record.id as string, status === 0 ? 1 : 0)}
          />
        );
      },
      valueType: 'select',
      valueEnum: {
        0: {
          text: '禁用',
        },
        1: {
          text: '启用',
        },
      },
    },
    {
      title: '创建时间',
      width: 135,
      search: false,
      dataIndex: 'createTime',
    },
    {
      title: '操作',
      search: false,
      fixed: 'right',
      width: 90,
      render: (_, record) => (
        <Space>
          <ModalFormEdit refresh={actionRef.current?.reset} initialData={record} />
          <Popconfirm title="确定删除该商户账号？" onConfirm={handleDelete.bind(null, record.id)}>
            <a>删除</a>
          </Popconfirm>
        </Space>
      ),
    },
  ];

  useEffect(() => {
    if (initialData.mchId) {
      formRef.current?.setFieldsValue({ mchId: Number(initialData.mchId) });
      formRef.current?.submit();
    }
  }, [initialData]);

  return (
    <ProTable
      {...PROTABLE_DEFAULT_SETTING}
      pagination={false}
      scroll={{ x: 1220, y: 'calc(100vh - 210px)' }}
      columns={columns}
      request={requestPaymentChannelList}
      actionRef={actionRef}
      formRef={formRef}
    />
  );
}

export default ChannelPayment;
