import { useRef } from 'react';
import { Button, message, Space, Alert, Table, Popconfirm } from 'antd';
import type { ColumnsType } from 'antd/lib/table';
import { addMerchantLoginIp, deleteMerchantLoginIp } from '@/services/merchant/merchant';
import { PlusOutlined } from '@ant-design/icons';
import { ModalForm, ProFormDigit, ProFormText, ProFormTextArea } from '@ant-design/pro-form';
import type { ProFormInstance } from '@ant-design/pro-form';

interface IProps {
  data: API.MerchantItem;
  onCancel: () => void;
  refresh: () => void;
}

const formItemLayout = {
  labelCol: {
    span: 4,
  },
  wrapperCol: {
    span: 18,
  },
};

function FormCommon({ data, refresh }: IProps) {
  const formRef = useRef<ProFormInstance>();

  const handleDelete = async (id: string) => {
    const res = await deleteMerchantLoginIp(id);
    if (res?.code === '0000') {
      message.success('删除成功');
      refresh();
    }
  };

  const columns: ColumnsType<API.ChannelFlowItem> = [
    {
      title: 'IP',
      dataIndex: 'ip',
      width: 135,
    },
    {
      title: '过期时间',
      dataIndex: 'expireTime',
      width: 180,
    },
    {
      title: '备注',
      dataIndex: 'memo',
      ellipsis: true,
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      width: 135,
    },
    {
      title: '操作人',
      dataIndex: 'createOperatorName',
      width: 80,
    },
    {
      title: '操作',
      width: 60,
      render: (_, record) => {
        return (
          <Popconfirm title="确定删除？" onConfirm={handleDelete.bind(null, record.id)}>
            <a>删除</a>
          </Popconfirm>
        );
      },
    },
  ];

  const handleFinish = async (values: any) => {
    const params = { mchId: data.id, ...values };
    const res = await addMerchantLoginIp(params);
    if (res.code === '0000') {
      refresh?.();
      message.success(`添加登录IP成功`);
      formRef.current?.resetFields();
      return true;
    }
    return false;
  };

  return (
    <div>
      <Space direction="vertical" style={{ width: '100%' }}>
        <Alert message="不配置商户登录IP，则默认允许商户在任意IP登录" type="warning" />
        <div className="text-right">
          <ModalForm
            {...formItemLayout}
            formRef={formRef}
            title="添加登录IP"
            layout="horizontal"
            width={520}
            trigger={
              <Button type="primary" icon={<PlusOutlined />} size="small">
                添加登录IP
              </Button>
            }
            onFinish={handleFinish}
          >
            <ProFormText
              name="loginIp"
              label="登录IP"
              placeholder="请输入登录IP"
              rules={[
                {
                  required: true,
                  message: '请输入登录IP!',
                },
              ]}
            />
            <ProFormDigit
              name="validDay"
              label="有效期"
              placeholder="请输入有效期"
              min={1}
              max={30}
              fieldProps={{ addonAfter: '天' }}
            />
            <ProFormTextArea name="memo" label="备注" placeholder="请输入备注" />
          </ModalForm>
        </div>
        <Table
          scroll={{ y: 'calc(100vh - 340px)' }}
          dataSource={data.loginIps}
          columns={columns}
          size="small"
        />
        <div className="text-right" />
        <div className="text-right" />
      </Space>
    </div>
  );
}

export default FormCommon;
