import { useEffect, useRef } from 'react';
import ProForm, { ProFormDigit, ProFormRadio, ProFormSelect } from '@ant-design/pro-form';
import type { ProFormInstance } from '@ant-design/pro-form';
import { Button, message, Row, Col, Space, Modal } from 'antd';
import InputArray from '@/components/InputArray';
import { editMerchantSafe, resetMerchantToken } from '@/services/merchant/merchant';
import MchKeyItem from './MchKeyItem';
import TokenItem from './TokenItem';

interface IProps {
  data: API.MerchantItem;
  onCancel: () => void;
  refresh: () => void;
}

const formItemLayout = {
  labelCol: {
    span: 4,
  },
  wrapperCol: {
    span: 18,
  },
};
const subFormItemLayout = {
  labelCol: {
    span: 8,
  },
  wrapperCol: {
    span: 14,
  },
};

function FormCommon({ data, onCancel, refresh }: IProps) {
  const formRef = useRef<ProFormInstance>();

  useEffect(() => {
    if (data) {
      formRef.current?.setFieldsValue(data);
    }
  }, [data]);

  const handleFinish = async (values: any) => {
    const params = { id: data.id, ...values };
    const res = await editMerchantSafe(params);
    if (res.code === '0000') {
      refresh?.();
      message.success(`编辑成功`);
      return true;
    }
    return false;
  };

  const handleReset = () => {
    Modal.confirm({
      title: '确认重置该商户秘钥？',
      okText: '确认',
      cancelText: '取消',
      onOk: async () => {
        const res = await resetMerchantToken(data.id);
        if (res.code === '0000') {
          refresh?.();
          message.success(`重置商户秘钥成功`);
        }
      },
    });
  };

  return (
    <div>
      <ProForm
        {...formItemLayout}
        formRef={formRef}
        layout="horizontal"
        initialValues={data}
        onFinish={handleFinish}
        submitter={{
          searchConfig: { submitText: '确认' },
          render: (props, doms) => {
            return (
              <Space style={{ width: '100%', justifyContent: 'flex-end' }}>
                <Button onClick={onCancel}>取消</Button>
                {doms[1]}
              </Space>
            );
          },
        }}
        // onReset={handleReset}
      >
        <Row>
          <Col span={12}>
            <ProForm.Item {...subFormItemLayout} name="mchKey" label="商户 key">
              <MchKeyItem />
            </ProForm.Item>
          </Col>
          <Col span={11}>
            <ProFormSelect
              {...subFormItemLayout}
              name="algorithm"
              label="加密算法"
              placeholder="请选择加密算法"
              valueEnum={{
                md5: 'md5',
                sha1: 'sha1',
                sha256: 'sha256',
                rsa: 'rsa',
              }}
              // rules={[{ required: true, message: '请选择货币单位！' }]}
            />
          </Col>
        </Row>
        <Row>
          <Col span={24}>
            <ProForm.Item name="apiToken" label="商户密钥">
              <TokenItem handleReset={handleReset} />
            </ProForm.Item>
          </Col>
        </Row>
        <Row>
          <Col span={24}>
            <ProForm.Item name="invokeIps" label="调用IP">
              <InputArray label="调用IP" />
            </ProForm.Item>
          </Col>
        </Row>
        <Row>
          <Col span={12}>
            <ProFormRadio.Group
              {...subFormItemLayout}
              name="paymentUserIp"
              label="下单用户IP"
              options={[
                {
                  label: '必传',
                  value: true,
                },
                {
                  label: '非必传',
                  value: false,
                },
              ]}
            />
          </Col>
          <Col span={11}>
            <ProFormDigit
              {...subFormItemLayout}
              name="onlineTerminalCount"
              min={1}
              max={10}
              fieldProps={{ addonAfter: '台设备' }}
              placeholder="可同时在线"
              label="可同时在线"
            />
          </Col>
        </Row>
      </ProForm>
    </div>
  );
}

export default FormCommon;
