import { useRef } from 'react';
import { addMerchant, updateMerchant } from '@/services/merchant/merchant';
import { PlusOutlined } from '@ant-design/icons';
import { DrawerForm, ProFormSelect, ProFormSwitch, ProFormText } from '@ant-design/pro-form';
import type { ProFormInstance } from '@ant-design/pro-form';
import { Button, message, Modal } from 'antd';
import AccountInfo from './AccountInfo';
import styles from '../style.less';
import { EDIT_TEXT, EDIT_TYPE } from '@/constants';
import { requestProxyOptions } from '@/services/commons';

const formItemLayout = {
  labelCol: {
    style: {
      width: '104px',
    },
  },
};

interface IProps {
  refresh?: () => void | undefined;
  type?: string;
  data?: any;
}

function ModalFormAdd({ refresh, type = EDIT_TYPE.ADD, data }: IProps) {
  const formRef = useRef<ProFormInstance>();

  const showAccoutInfo = (info: API.AccountInfo) => {
    Modal.info({
      className: styles.infoModal,
      width: 600,
      content: <AccountInfo data={info} />,
    });
  };

  const handleFinish = async (values: API.MerchantItem) => {
    let res;
    if (type === EDIT_TYPE.EDIT) {
      res = await updateMerchant({
        ...data,
        ...values,
        status: values.status ? 1 : 0,
      });
    } else {
      res = await addMerchant({ ...values });
    }
    if (res.code === '0000') {
      refresh?.();
      formRef.current?.resetFields();
      message.success(`${EDIT_TEXT[type]}成功`);
      if (type === EDIT_TYPE.ADD) {
        showAccoutInfo(res.data);
      }
      return true;
    }
    return false;
  };

  const handleVisibleChange = (visible: boolean) => {
    if (visible) {
      if (type === EDIT_TYPE.EDIT) {
        formRef.current?.setFieldsValue({ ...data, status: !!data.status });
      }
    }
  };

  return (
    <div>
      <DrawerForm
        {...formItemLayout}
        formRef={formRef}
        title={`${EDIT_TEXT[type]}商户`}
        layout="horizontal"
        width={720}
        trigger={
          type === EDIT_TYPE.ADD ? (
            <Button icon={<PlusOutlined />} type="primary">
              创建商户
            </Button>
          ) : (
            <a>编辑</a>
          )
        }
        onFinish={handleFinish}
        onVisibleChange={handleVisibleChange}
        initialValues={type === EDIT_TYPE.EDIT ? { ...data, status: !!data.status } : {}}
      >
        <ProFormText
          name="mchName"
          placeholder="请输入商户名称"
          label="商户名称"
          width={500}
          rules={[{ required: true, message: '请输入商户名称!' }]}
        />
        <ProFormText
          name="loginName"
          label="登录名称"
          disabled={type === EDIT_TYPE.EDIT}
          placeholder="请输入登录名称"
          width={500}
          rules={[
            {
              required: true,
              message: '请输入登录名称！',
            },
          ]}
        />
        <ProFormSelect
          width={500}
          name="proxyId"
          placeholder="请选择上级代理"
          label="上级代理"
          request={requestProxyOptions}
        />
        <ProFormText name="groupName" label="分组名称" placeholder="请输入分组名称" width={500} />
        {type === EDIT_TYPE.EDIT && (
          <ProFormSwitch
            name="status"
            label="状态"
            checkedChildren="启用"
            unCheckedChildren="禁用"
          />
        )}
      </DrawerForm>
    </div>
  );
}

export default ModalFormAdd;
