import { EDIT_TYPE, PROTABLE_DEFAULT_SETTING, ROUTER_ENUM } from '@/constants';
import ProTable from '@ant-design/pro-table';
import type { ActionType } from '@ant-design/pro-table';
import type { ProColumns } from '@ant-design/pro-table';
import {
  closeMerchantGoogleAuth,
  deleteMerchant,
  requestMerchantList,
  resetMerchantPwd,
  switchMerchantStatus,
} from '@/services/merchant/merchant';
import { message, Modal, Popconfirm, Space, Switch } from 'antd';
import { useRef, useReducer } from 'react';
import type { ReactNode } from 'react';
import { history } from 'umi';
import Ip from '@/components/IP';
import { requestProxyOptions } from '@/services/commons';
import type { ICountReducer } from '@/reducers/modalReducer';
import modalReducer from '@/reducers/modalReducer';
import ModalFormAdd from './components/ModalFormAdd';
import AccountInfo from './components/AccountInfo';
import ModalFormSync from './components/ModalFormSync';
import styles from './style.less';
import ModalFormEdit from './components/ModalFormEdit';
import ModalFormClear from './components/ModalFormClear';

const showAccoutInfo = (info: API.AccountInfo) => {
  Modal.info({
    className: styles.infoModal,
    width: 600,
    content: <AccountInfo data={info} />,
  });
};

// 重置密码
const handleReset = async (id: string) => {
  const res = await resetMerchantPwd(id);
  if (res?.code === '0000') {
    showAccoutInfo(res.data);
    message.success('重置成功');
  }
};

function MerchantList() {
  const actionRef = useRef<ActionType>();
  const [state, dispatch] = useReducer<ICountReducer>(modalReducer, {
    visible: false,
    initialData: {},
  });

  const closeGoogleAuth = async (id: string) => {
    Modal.confirm({
      title: '关闭谷歌验证',
      content: '确认关闭该商户谷歌验证？',
      okText: '确认',
      cancelText: '取消',
      onOk: async () => {
        const res = await closeMerchantGoogleAuth(id);
        if (res?.code === '0000') {
          actionRef.current?.reload();
          message.success('关闭成功');
        }
      },
    });
  };

  const handleDelete = async (id: string) => {
    const res = await deleteMerchant(id);
    if (res?.code === '0000') {
      actionRef.current?.reload();
      message.success('删除成功');
    }
  };

  // 切换状态
  const switchStatus = async (id: string, status: number) => {
    const text = status === 0 ? '禁用' : '启用';
    const res = await switchMerchantStatus(id, status);
    if (res?.code === '0000') {
      actionRef.current?.reload?.();
      message.success(`${text}成功`);
    }
  };

  // 查看通道费率
  const showFee = (mchId: string) => {
    history.push({
      pathname: ROUTER_ENUM.merchantChannel,
      query: { mchId },
    });
  };

  const showEdit = (record: API.MerchantItem) => {
    dispatch({
      type: 'showModal',
      payload: {
        initialData: record,
      },
    });
  };

  const columns: ProColumns<API.MerchantItem>[] = [
    {
      title: '商户ID',
      dataIndex: 'id',
      width: 65,
      order: 1,
    },
    {
      title: '分组名称',
      width: 110,
      dataIndex: 'groupName',
    },
    {
      title: '商户名称',
      width: 110,
      dataIndex: 'mchName',
      order: 1,
    },
    {
      title: '登录名称',
      width: 110,
      dataIndex: 'loginName',
      order: 1,
    },
    {
      title: '代理',
      width: 110,
      dataIndex: 'proxyId',
      valueType: 'select',
      fieldProps: {
        showSearch: true,
      },
      request: requestProxyOptions,
      order: 1,
      hideInTable: true,
    },
    {
      title: '代理名称',
      width: 110,
      dataIndex: 'proxyName',
      search: false,
      order: 1,
    },
    {
      title: '状态',
      dataIndex: 'status',
      width: 80,
      render: (_, record) => {
        const { status } = record;
        return (
          <Switch
            // disabled={!googleStatus}
            checkedChildren="启用"
            unCheckedChildren="禁用"
            checked={!!status}
            onClick={switchStatus.bind(null, record.id as string, status === 0 ? 1 : 0)}
          />
        );
      },
      valueType: 'select',
      valueEnum: {
        0: {
          text: '禁用',
        },
        1: {
          text: '启用',
        },
      },
    },
    {
      title: '账户余额',
      width: 150,
      search: false,
      dataIndex: 'mchBalance',
      render: (_) => <strong className="orange">{_}</strong>,
    },

    {
      title: '通道费率',
      width: 75,
      search: false,
      dataIndex: 'productCode',
      render: (_, record) => <a onClick={showFee.bind(null, record.id || '')}>查看</a>,
    },
    {
      title: '谷歌验证',
      search: false,
      width: 80,
      dataIndex: 'googleStatus',
      render: (googleStatus: ReactNode, record: API.MerchantItem) => {
        return (
          <Switch
            disabled={!googleStatus}
            checkedChildren="开启"
            unCheckedChildren="关闭"
            checked={googleStatus as boolean}
            onClick={closeGoogleAuth.bind(null, record.id)}
          />
        );
      },
    },
    {
      title: '上次登录时间',
      width: 135,
      search: false,
      dataIndex: 'lastLoginTime',
    },
    {
      title: '上次登录IP',
      width: 138,
      search: false,
      dataIndex: 'lastLoginIp',
      render: (_, record) => <Ip ip={record.lastLoginIp} />,
    },
    {
      title: '创建时间',
      width: 135,
      search: false,
      dataIndex: 'createTime',
    },
    {
      title: '操作',
      width: 210,
      search: false,
      fixed: 'right',
      render: (_, record) => (
        <Space>
          <a onClick={showEdit.bind(null, record)}>编辑</a>
          <Popconfirm title="确定重置该商户密码？" onConfirm={handleReset.bind(null, record.id)}>
            <a>重置密码</a>
          </Popconfirm>
          <ModalFormClear data={record} refresh={actionRef.current?.reload} />
          <Popconfirm title="确定删除该商户？" onConfirm={handleDelete.bind(null, record.id)}>
            <a>删除</a>
          </Popconfirm>
        </Space>
      ),
    },
  ];
  return (
    <>
      <ProTable
        {...PROTABLE_DEFAULT_SETTING}
        scroll={{ x: 1600, y: 'calc(100vh - 340px)' }}
        columns={columns}
        request={requestMerchantList}
        form={{
          colon: false,
        }}
        actionRef={actionRef}
        search={{
          labelWidth: 'auto',
        }}
        defaultSize="small"
        toolBarRender={() => [
          <ModalFormAdd refresh={actionRef.current?.reset} type={EDIT_TYPE.ADD} />,
          <ModalFormSync refresh={actionRef.current?.reset} />,
        ]}
      />
      <Modal
        width={880}
        className={styles.modalFormEdit}
        visible={state.visible}
        onCancel={() => {
          dispatch({ type: 'closeModal' });
        }}
        title={`商户配置：${state.initialData?.mchName}`}
        destroyOnClose
        maskClosable={false}
        footer={null}
      >
        <ModalFormEdit
          data={state.initialData as API.MerchantItem}
          onCancel={() => {
            dispatch({ type: 'closeModal' });
          }}
          refresh={actionRef.current?.reload}
        />
      </Modal>
    </>
  );
}

export default MerchantList;
