import { useRef } from 'react';
import { EDIT_TYPE, PROTABLE_DEFAULT_SETTING } from '@/constants';
import ProTable from '@ant-design/pro-table';
import type { ActionType } from '@ant-design/pro-table';
import type { ProColumns } from '@ant-design/pro-table';
import { message, Modal, Popconfirm, Space, Switch, Tooltip } from 'antd';
import {
  closeProxyGoogleAuth,
  deleteProxy,
  requestProxyList,
  resetProxyPwd,
  switchProxyStatus,
} from '@/services/merchant/proxy';
import ModalFormAdd from './components/ModalFormAdd';
import AccountInfo from './components/AccountInfo';
import Ip from '@/components/IP';
import { InfoCircleOutlined } from '@ant-design/icons';
import styles from './style.less';
import InfoTable from './components/InfoTable';
import RateConfig from './components/RateConfig';

function ProxyList() {
  const actionRef = useRef<ActionType>();

  const closeGoogleAuth = async (id: string) => {
    Modal.confirm({
      title: '关闭谷歌验证',
      content: '确认关闭该代理谷歌验证？',
      okText: '确认',
      cancelText: '取消',
      onOk: async () => {
        const res = await closeProxyGoogleAuth(id);
        if (res?.code === '0000') {
          actionRef.current?.reload();
          message.success('关闭成功');
        }
      },
    });
  };

  const handleDelete = async (id: string) => {
    const res = await deleteProxy(id);
    if (res?.code === '0000') {
      actionRef.current?.reload();
      message.success('删除成功');
    }
  };

  // 切换状态
  const switchStatus = async (id: string, status: number) => {
    const text = status === 0 ? '禁用' : '启用';
    const res = await switchProxyStatus(id, status);
    if (res?.code === '0000') {
      actionRef.current?.reload?.();
      message.success(`${text}成功`);
    }
  };

  const showAccoutInfo = (info: API.AccountInfo) => {
    Modal.info({
      className: styles.infoModal,
      width: 500,
      content: <AccountInfo data={info} />,
    });
  };

  // 重置密码
  const handleReset = async (id: string) => {
    const res = await resetProxyPwd(id);
    if (res?.code === '0000') {
      showAccoutInfo(res.data);
      message.success('重置成功');
    }
  };

  // 展示下级商户信息
  const showInfo = (id: string) => {
    Modal.info({
      okText: '知道了',
      className: styles.infoModal,
      width: 700,
      content: <InfoTable id={id} />,
    });
  };

  // 展示费率配置
  const showRateConfig = (id: string) => {
    Modal.info({
      okText: '知道了',
      className: `${styles.infoModal} ${styles.rateModal}`,
      width: 850,
      style: { top: 30 },
      content: <RateConfig id={id} />,
    });
  };

  const columns: ProColumns<API.ProxyItem>[] = [
    {
      title: '代理ID',
      dataIndex: 'id',
      width: 70,
      // order: 4,
    },
    {
      title: '代理名称',
      width: 110,
      dataIndex: 'proxyName',
      // order: 4,
    },
    {
      title: '登录名称',
      width: 110,
      dataIndex: 'loginName',
    },
    {
      title: '状态',
      dataIndex: 'status',
      width: 80,
      render: (_, record) => {
        const { status } = record;
        return (
          <Switch
            // disabled={!googleStatus}
            checkedChildren="启用"
            unCheckedChildren="禁用"
            checked={!!status}
            onClick={switchStatus.bind(null, record.id as string, status === 0 ? 1 : 0)}
          />
        );
      },
      valueType: 'select',
      valueEnum: {
        0: {
          text: '禁用',
        },
        1: {
          text: '启用',
        },
      },
    },
    {
      title: '账户余额',
      width: 150,
      search: false,
      dataIndex: 'proxyBalance',
      render: (_) => <strong className="orange">{_}</strong>,
    },

    {
      title: '通道费率',
      width: 80,
      search: false,
      dataIndex: 'productCode',
      render: (_, record) => (
        <Space>
          <a onClick={showRateConfig.bind(null, record.id as string)}>查看</a>
        </Space>
      ),
    },
    {
      title: '下级商户个数',
      width: 110,
      search: false,
      dataIndex: 'merchantCount',
      render: (_, record) => {
        return (
          <div className="cursor-pointer flex-center">
            <span>{_}个</span>
            {_ !== 0 && (
              <span className={styles.info} onClick={showInfo.bind(null, record.id as string)}>
                <Tooltip title="详情">
                  <InfoCircleOutlined />
                </Tooltip>
              </span>
            )}
          </div>
        );
      },
    },
    {
      title: '谷歌验证',
      search: false,
      width: 80,
      dataIndex: 'googleStatus',
      render: (googleStatus, record) => {
        return (
          <Switch
            disabled={!googleStatus}
            checkedChildren="开启"
            unCheckedChildren="关闭"
            checked={googleStatus as boolean}
            onClick={closeGoogleAuth.bind(null, record.id)}
          />
        );
      },
    },
    {
      title: '上次登录时间',
      search: false,
      width: 135,
      dataIndex: 'lastLoginTime',
    },
    {
      title: '上次登录IP',
      search: false,
      width: 138,
      dataIndex: 'lastLoginIp',
      render: (_, record) => <Ip ip={record.lastLoginIp} />,
    },
    {
      title: '创建时间',
      search: false,
      width: 135,
      dataIndex: 'createTime',
    },
    {
      title: '操作',
      width: 155,
      search: false,
      fixed: 'right',
      render: (_, record: API.ProxyItem) => (
        <Space>
          <ModalFormAdd refresh={actionRef.current?.reset} type={EDIT_TYPE.EDIT} data={record} />
          <Popconfirm title="确定重置该代理密码？" onConfirm={handleReset.bind(null, record.id)}>
            <a>重置密码</a>
          </Popconfirm>
          <Popconfirm title="确定删除该代理？" onConfirm={handleDelete.bind(null, record.id)}>
            <a>删除</a>
          </Popconfirm>
        </Space>
      ),
    },
  ];
  return (
    <ProTable
      {...PROTABLE_DEFAULT_SETTING}
      scroll={{ x: 1500, y: 'calc(100vh - 340px)' }}
      columns={columns}
      request={requestProxyList}
      form={{
        colon: false,
      }}
      actionRef={actionRef}
      search={{
        labelWidth: 'auto',
      }}
      defaultSize="small"
      toolBarRender={() => [<ModalFormAdd refresh={actionRef.current?.reset} />]}
    />
  );
}

export default ProxyList;
