import { PROTABLE_DEFAULT_SETTING } from '@/constants';
import { useState, useRef, useEffect } from 'react';
import ProTable from '@ant-design/pro-table';
import type { ActionType, ProColumns } from '@ant-design/pro-table';
import {
  auditPayment,
  auditPaymentBatch,
  closePayment,
  closePaymentBatch,
  queryPayment,
  requestPaymentList,
  retryMerchantNotifyBatch,
} from '@/services/merchant/payment';
import { Button, Drawer, message, Modal, Popconfirm, Space } from 'antd';
import { getBaseColumns } from './constants';
import HeaderTitle from './components/HeaderTitle';
import Info from './components/Info';
import ChannelInfo from './components/ChannelInfo';
import StatusInfo from './components/StatusInfo';
import ChannelNotify from './components/ChannelNotify';
import ModalFormReplenishment from './components/ModalFormReplenishment';
import ModalFormBatchReplenishment from './components/ModalFormBatchReplenishment';
import OrderResult from './components/OrderResult';
import type { ProFormInstance } from '@ant-design/pro-form';
import AutoRefresh from './components/AutoRefresh';

// 批量成功提示
export const handleBatchSuccess = (
  msg: string,
  data: { total: number; fail: number; success: number },
) => {
  message.success(
    `${msg}：共${data.total || 0}笔订单，成功${data.success || 0}笔，失败${data.fail || 0}笔`,
    4,
  );
};

export interface IExtraData {
  channelFee: string;
  mchBalance: string;
  mchFee: string;
  proxyFee: string;
  serialIncome: string;
  totalAmount: string;
}

function Payment({ location: { query } }: any) {
  const actionRef = useRef<ActionType>();
  const formRef = useRef<ProFormInstance>();

  const [extraData, setExtraData] = useState<Partial<IExtraData>>({});

  const [curSerialOrderNo, setCurSerialOrderNo] = useState<string>('');
  const [curNotifyUrl, setCurNotifyUrl] = useState<string>('');
  // 订单详情
  const [infoVisible, setInfoVisible] = useState<boolean>(false);

  // 通道调用信息
  const [channelVisible, setChannelVisible] = useState<boolean>(false);

  // 商户回调状态
  const [statusVisible, setStatusVisible] = useState<boolean>(false);

  // 通道回调状态
  const [notifyStatusVisible, setNotifyStatusVisible] = useState<boolean>(false);

  // 查询订单结果状态
  const [orderResultVisible, setOrderResultVisible] = useState<boolean>(false);
  const [orderResult, setOrderResult] = useState<Partial<API.OrderResultItem>>({});
  const [queryOrderBtnLoading, setQueryOrderBtnLoading] = useState<boolean>(false);

  useEffect(() => {
    if (query.mchOrderNo) {
      formRef.current?.setFieldsValue({ mchOrderNo: query.mchOrderNo });
      formRef.current?.submit();
    }
  }, [query.mchOrderNo]);

  // 关闭订单
  const handleClose = async (serialOrderNo: string) => {
    const res = await closePayment(serialOrderNo);
    if (res?.code === '0000') {
      actionRef.current?.reload();
      message.success('关闭成功');
    }
  };

  // 查单
  const handleQuery = async (serialOrderNo: string) => {
    if (queryOrderBtnLoading) {
      return;
    }
    setQueryOrderBtnLoading(true);
    const res = await queryPayment(serialOrderNo);
    setQueryOrderBtnLoading(false);
    if (res?.code === '0000') {
      setOrderResultVisible(true);
      setOrderResult(res.data);
      actionRef.current?.reload();
    }
  };

  // 审计订单
  const handleAudit = async (serialOrderNo: string) => {
    Modal.confirm({
      title: '确认审计该订单？',
      okText: '确认',
      cancelText: '取消',
      onOk: async () => {
        const res = await auditPayment(serialOrderNo);
        if (res?.code === '0000') {
          actionRef.current?.reload();
          message.success('审计成功');
          return true;
        }
        throw new Error('接口请求失败');
      },
    });
  };

  // 批量审计订单
  const handleBatchAudit = async (serialOrderNos: (string | number)[]) => {
    Modal.confirm({
      title: '确认批量审计？',
      content: `订单个数：${serialOrderNos.length}`,
      okText: '确认',
      cancelText: '取消',
      onOk: async () => {
        const res = await auditPaymentBatch(serialOrderNos);
        if (res?.code === '0000') {
          actionRef.current?.reload();
          handleBatchSuccess('批量审计执行成功', res?.data);
          return true;
        }
        throw new Error('接口请求失败');
      },
    });
  };

  const showInfo = (serialOrderNo: string) => {
    setCurSerialOrderNo(serialOrderNo);
    setInfoVisible(true);
  };

  const showChannelInfo = (serialOrderNo: string) => {
    setCurSerialOrderNo(serialOrderNo);
    setChannelVisible(true);
  };

  const showNotifyStatus = (serialOrderNo: string, notifyUrl: string) => {
    setCurNotifyUrl(notifyUrl);
    setCurSerialOrderNo(serialOrderNo);
    setStatusVisible(true);
  };

  const showChannelNotify = (serialOrderNo: string) => {
    setCurSerialOrderNo(serialOrderNo);
    setNotifyStatusVisible(true);
  };

  // 批量回调
  const handleBatchRetryNotify = (serialOrderNos: (string | number)[]) => {
    Modal.confirm({
      title: '确认批量回调？',
      content: `订单个数：${serialOrderNos.length}`,
      okText: '确认',
      cancelText: '取消',
      onOk: async () => {
        const res = await retryMerchantNotifyBatch(serialOrderNos);
        if (res?.code === '0000') {
          actionRef.current?.reload();
          handleBatchSuccess('批量回调执行成功', res?.data);
          return true;
        }
        throw new Error('接口请求失败');
      },
    });
  };

  // 批量关闭
  const handleBatchClose = (serialOrderNos: (string | number)[]) => {
    Modal.confirm({
      title: '确认批量关闭？',
      content: `订单个数：${serialOrderNos.length}`,
      okText: '确认',
      cancelText: '取消',
      onOk: async () => {
        const res = await closePaymentBatch(serialOrderNos);
        if (res?.code === '0000') {
          actionRef.current?.reload();
          handleBatchSuccess('批量关闭执行成功', res?.data);
          return true;
        }
        throw new Error('接口请求失败');
      },
    });
  };

  const columns: ProColumns<API.PaymentItem>[] = [
    ...getBaseColumns({
      showChannelInfo,
      showNotifyStatus,
      showChannelNotify,
      handleAudit,
    }),
    {
      title: '操作',
      search: false,
      fixed: 'right',
      width: 200,
      render: (_, record) => (
        <div>
          <Button
            type="link"
            size="small"
            onClick={showInfo.bind(null, record.serialOrderNo || '')}
          >
            详情
          </Button>
          {record.statusCode === 0 ? (
            <Popconfirm title="确定关闭？" onConfirm={handleClose.bind(null, record.serialOrderNo)}>
              <Button type="link" size="small" disabled={record.statusCode !== 0}>
                关闭
              </Button>
            </Popconfirm>
          ) : (
            <Button type="link" size="small" disabled={record.statusCode !== 0}>
              关闭
            </Button>
          )}
          {record.statusCode !== 1 ? (
            <ModalFormReplenishment data={record} refresh={actionRef.current?.reload} />
          ) : (
            <Button type="link" size="small" disabled={record.statusCode === 1}>
              补单
            </Button>
          )}
          <Button type="link" size="small" onClick={handleQuery.bind(null, record.serialOrderNo)}>
            查单
          </Button>
        </div>
      ),
    },
  ];

  return (
    <>
      <ProTable
        {...PROTABLE_DEFAULT_SETTING}
        scroll={{ x: 2200, y: 'calc(100vh - 340px)' }}
        columns={columns}
        request={requestPaymentList.bind(null, setExtraData)}
        formRef={formRef}
        actionRef={actionRef}
        rowKey="serialOrderNo"
        rowSelection={{
          fixed: true,
          // selections: [Table.SELECTION_ALL, Table.SELECTION_INVERT],
        }}
        tableAlertRender={({ selectedRowKeys, onCleanSelected }) => (
          <Space size={24}>
            <span>
              已选 {selectedRowKeys.length} 项
              <a style={{ marginLeft: 8 }} onClick={onCleanSelected}>
                取消选择
              </a>
            </span>
          </Space>
        )}
        tableAlertOptionRender={({ selectedRowKeys }) => {
          return (
            <Space size={16}>
              <a onClick={handleBatchRetryNotify.bind(null, selectedRowKeys)}>批量回调</a>
              <a onClick={handleBatchAudit.bind(null, selectedRowKeys)}>批量审计</a>
              <a onClick={handleBatchClose.bind(null, selectedRowKeys)}>批量关闭</a>
              <ModalFormBatchReplenishment
                serialOrderNos={selectedRowKeys}
                refresh={actionRef.current?.reload}
              />
            </Space>
          );
        }}
        headerTitle={<HeaderTitle data={extraData} />}
        toolBarRender={() => [<AutoRefresh refresh={actionRef.current?.reload} key="refresh" />]}
      />
      <Drawer
        destroyOnClose
        width={700}
        visible={infoVisible}
        title="订单详情"
        onClose={() => setInfoVisible(false)}
      >
        <Info serialOrderNo={curSerialOrderNo} />
      </Drawer>
      <Drawer
        destroyOnClose
        width={700}
        visible={channelVisible}
        title="通道调用信息"
        onClose={() => setChannelVisible(false)}
      >
        <ChannelInfo serialOrderNo={curSerialOrderNo} />
      </Drawer>
      <Drawer
        destroyOnClose
        width={700}
        visible={statusVisible}
        title="商户回调记录"
        onClose={() => setStatusVisible(false)}
      >
        <StatusInfo serialOrderNo={curSerialOrderNo} notifyUrl={curNotifyUrl} />
      </Drawer>
      <Drawer
        destroyOnClose
        width={700}
        visible={notifyStatusVisible}
        title="通道回调记录"
        onClose={() => setNotifyStatusVisible(false)}
      >
        <ChannelNotify serialOrderNo={curSerialOrderNo} />
      </Drawer>
      <Drawer
        destroyOnClose
        width={700}
        visible={orderResultVisible}
        title="查询订单结果"
        onClose={() => setOrderResultVisible(false)}
      >
        <OrderResult orderResult={orderResult} />
      </Drawer>
    </>
  );
}

export default Payment;
