import { useState, useEffect } from 'react';
import { RedoOutlined } from '@ant-design/icons';
import { useRequest } from 'umi';
import { Button, message, Input, Spin, Steps } from 'antd';
import 'codemirror/lib/codemirror';
import 'codemirror/lib/codemirror.css';
import 'codemirror/mode/javascript/javascript'; // js
import 'codemirror/theme/idea.css';
import { Controlled as CodeMirror } from 'react-codemirror2';
import { getMerchantNotifyLog, retryMerchantNotify } from '@/services/merchant/transfer';
import Copy from '@/components/Copy';
import { formatJSON } from '@/utils';
import styles from '../style.less';

interface IProps {
  serialOrderNo: string;
  notifyUrl: string;
}

const { Step } = Steps;

function StatusInfo({ serialOrderNo, notifyUrl }: IProps) {
  const {
    data = [],
    loading,
    refresh,
  } = useRequest<{
    data: API.NotifyLog[];
  }>(getMerchantNotifyLog.bind(null, serialOrderNo));

  const [retryLoading, setRetryLoading] = useState<boolean>(false);
  const [curNotifyUrl, setCurNotifyUrl] = useState<string>('');

  useEffect(() => {
    setCurNotifyUrl(notifyUrl);
  }, [notifyUrl]);

  const retry = async () => {
    setRetryLoading(true);
    const res = await retryMerchantNotify(serialOrderNo, curNotifyUrl);
    setRetryLoading(false);
    if (res?.code === '0000') {
      refresh();
      message.success('重试成功');
    }
  };

  return (
    <Spin spinning={loading}>
      <div className={styles.info}>
        <div className={styles.header}>
          <div className={styles.row}>
            回调次数：<strong>{data.length}</strong>
          </div>
          <div className={styles.row}>
            回调地址：
            <Input.Group compact className={styles.notifyUrl}>
              <Input
                defaultValue={notifyUrl}
                onChange={({ target: { value } }) => setCurNotifyUrl(value)}
              />
              <Button type="primary" icon={<RedoOutlined />} onClick={retry} loading={retryLoading}>
                重试
              </Button>
            </Input.Group>
          </div>
        </div>
        {!!data.length && (
          <Steps direction="vertical">
            {data.map((item) => {
              const description = (
                <div>
                  <div className={styles.statusItem}>
                    <span className={styles.title}>回调地址：</span>
                    {item?.notifyUrl}
                  </div>
                  <div className={styles.statusItem}>
                    <span className={styles.title}>
                      <Copy text={item.notifyParam}>回调参数：</Copy>
                    </span>
                    <CodeMirror
                      className={styles.codeMirror}
                      value={formatJSON(item?.notifyParam || '')}
                      options={{
                        mode: 'application/json',
                        theme: 'idea',
                        lineNumbers: true,
                      }}
                      onBeforeChange={() => {}}
                    />
                  </div>
                  <div className={styles.statusItem}>
                    <span className={styles.title}>回调时间：</span>
                    {item?.notifyTime}
                  </div>
                  <div className={styles.statusItem}>
                    <span className={styles.title}>回调响应：</span>
                    {item?.notifyResult}
                  </div>
                  <div className={styles.statusItem}>
                    <span className={styles.title}>回调耗时：</span>
                    {item?.runTime}
                  </div>
                </div>
              );
              const status = item?.success ? 'finish' : 'error';
              const title = item?.success ? '回调成功' : '回调失败';
              return <Step status={status} title={title} description={description} />;
            })}
          </Steps>
        )}
      </div>
    </Spin>
  );
}

export default StatusInfo;
