import type { ProColumns } from '@ant-design/pro-table';
import { Badge, Tag, Tooltip } from 'antd';
import { requestMerchantOptions } from '@/services/commons';
import {
  CheckCircleOutlined,
  ClockCircleTwoTone,
  ExclamationCircleOutlined,
  InfoCircleOutlined,
  PushpinTwoTone,
} from '@ant-design/icons';
import { DEFAULT_DATE_RANGES, TIME_TYPE_ENUM } from '@/constants';
import TimeCountdown from '@/components/TimeCountdown';
import { requestChannelOptions, requestProductOptions } from '@/services/merchant/transfer';

// 交易状态
export const TRANSACTION_STATUS_MAP = {
  PROCESSING: {
    status: 'processing',
    text: '交易中',
  },
  SUCCESS: {
    status: 'success',
    text: '交易成功',
  },
  FAIL: {
    status: 'error',
    text: '交易失败',
  },
};

export const TRANSFER_STATUS_MAP = {
  0: {
    status: 'processing',
    text: '打款中',
  },
  1: {
    status: 'success',
    text: '打款成功',
  },
  2: {
    status: 'warning',
    text: '打款失败',
  },
  3: {
    status: 'default',
    text: '超时关闭',
  },
  4: {
    status: 'error',
    text: '下单失败',
  },
  5: {
    status: 'default',
    text: '手动关闭',
  },
};

export const PAYMENT_NOTIFY_STATUS_MAP = {
  0: {
    text: '待回调',
    status: 'processing',
  },
  1: {
    text: '回调成功',
    status: 'success',
  },
  2: {
    text: '回调失败',
    status: 'error',
  },
  3: {
    text: '无需回调',
    status: 'default',
  },
};

const showErrorMsg = (statusCode: number) => [2, 4].includes(statusCode);
interface IParams {
  showChannelInfo: (id: string) => void;
  showNotifyStatus: (id: string, notifyUrl: string) => void;
  showChannelNotify: (id: string) => void;
  handleAudit: (id?: string) => void;
}

const timeTypeEnum = {
  [TIME_TYPE_ENUM.ONE_MINUTE]: { text: '1分钟内' },
  [TIME_TYPE_ENUM.FIVE_MINUTES]: { text: '5分钟内' },
  [TIME_TYPE_ENUM.TEN_MINUTES]: { text: '10分钟内' },
  [TIME_TYPE_ENUM.HALF_HOUR]: { text: '30分钟内' },
  [TIME_TYPE_ENUM.ONE_HOUR]: { text: '1小时内' },
  [TIME_TYPE_ENUM.TODAY]: { text: '今天' },
  [TIME_TYPE_ENUM.YESTERDAY]: { text: '昨天' },
  [TIME_TYPE_ENUM.BEFORE_YESTERDAY]: { text: '前天' },
};

export const getBaseColumns = ({
  showChannelInfo,
  showNotifyStatus,
  showChannelNotify,
  handleAudit,
}: IParams) => {
  const BASE_COLUMNS: ProColumns<API.PaymentItem>[] = [
    {
      title: '商户交易号',
      dataIndex: 'mchOrderNo',
      order: 5,
      width: 320,
      copyable: true,
      ellipsis: true,
    },
    {
      title: '平台流水号',
      dataIndex: 'serialOrderNo',
      order: 4,
      width: 320,
      copyable: true,
      ellipsis: true,
    },
    {
      title: '通道订单号',
      dataIndex: 'channelOrderNo',
      order: 4,
      width: 320,
      copyable: true,
      ellipsis: true,
    },
    {
      title: '商户名称',
      dataIndex: 'mchName',
      width: 90,
      search: false,
    },
    {
      title: '订单金额',
      width: 100,
      search: false,
      dataIndex: 'amount',
      render: (_) => <strong className="orange">{_}</strong>,
    },
    {
      title: '打款金额',
      width: 100,
      search: false,
      dataIndex: 'realAmount',
      render: (text, record) =>
        text !== '-' ? (
          <strong className={`orange ${record.amount !== text ? 'error' : ''}`}>{text}</strong>
        ) : (
          text
        ),
    },
    {
      title: '商户费率',
      width: 90,
      ellipsis: true,
      search: false,
      dataIndex: 'mchFeeRate',
    },
    {
      title: '商户手续费',
      width: 100,
      search: false,
      dataIndex: 'mchFee',
    },
    {
      title: '大通道',
      ellipsis: true,
      width: 110,
      search: false,
      dataIndex: 'productName',
    },
    {
      title: '子通道',
      ellipsis: true,
      width: 130,
      search: false,
      dataIndex: 'channelName',
      render: (text, record) => {
        return (
          <a
            className={record.statusCode === 4 ? 'error' : ''}
            onClick={() => showChannelInfo(record.serialOrderNo || '')}
          >
            {text}
          </a>
        );
      },
    },

    {
      title: '通道回调',
      width: 150,
      ellipsis: true,
      dataIndex: 'channelNotify',
      render: (_, record) =>
        record.channelNotify ? (
          <a onClick={showChannelNotify.bind(null, record.serialOrderNo || '')}>{_}</a>
        ) : (
          '-'
        ),
    },
    {
      title: '商户回调',
      width: 90,
      dataIndex: 'notifyStatusCode',
      valueType: 'select',
      valueEnum: PAYMENT_NOTIFY_STATUS_MAP,
      render: (_, record) =>
        !isNaN(record.notifyStatusCode) &&
        PAYMENT_NOTIFY_STATUS_MAP[record.notifyStatusCode] && (
          <Badge
            status={PAYMENT_NOTIFY_STATUS_MAP[record.notifyStatusCode].status}
            text={
              [1, 2].includes(record.notifyStatusCode) ? (
                <a
                  className={`${record.notifyStatusCode === 2 ? 'error' : ''}`}
                  onClick={() =>
                    showNotifyStatus(record.serialOrderNo || '', record.notifyUrl || '')
                  }
                >
                  {PAYMENT_NOTIFY_STATUS_MAP[record.notifyStatusCode].text}
                </a>
              ) : (
                PAYMENT_NOTIFY_STATUS_MAP[record.notifyStatusCode].text
              )
            }
          />
        ),
    },
    {
      title: '订单状态',
      width: 125,
      dataIndex: 'statusCode',
      order: 8,
      render: (_, record) => {
        return (
          <span className="flex-center">
            {!isNaN(record.statusCode) && TRANSFER_STATUS_MAP[record.statusCode] && (
              <Tag color={TRANSFER_STATUS_MAP[record.statusCode].status}>
                {TRANSFER_STATUS_MAP[record.statusCode].text}
                {record.statusCode === 0 && (
                  <TimeCountdown
                    timeCount={(record.residualMillSecond || 0) + new Date().getTime()}
                  />
                )}
              </Tag>
            )}

            {/* 支付失败icon */}
            {showErrorMsg(record.statusCode) && (
              <Tooltip title={record.message}>
                <span className={TRANSFER_STATUS_MAP[record.statusCode].status}>
                  <InfoCircleOutlined />
                </span>
              </Tooltip>
            )}

            {/* 审计 */}
            {record.statusCode === 1 &&
              (record.audit ? (
                <Tooltip title="已审计">
                  <span className="success" style={{ marginRight: 8 }}>
                    <CheckCircleOutlined />
                  </span>
                </Tooltip>
              ) : (
                <Tooltip title="未审计，点击去审计">
                  <span
                    className="warning"
                    style={{ marginRight: 8, cursor: 'pointer' }}
                    onClick={handleAudit.bind(null, record.serialOrderNo)}
                  >
                    <ExclamationCircleOutlined />
                  </span>
                </Tooltip>
              ))}

            {/* 超时| 补单icon */}
            {record.statusCode === 1 && record.overtime && (
              <Tooltip title="超时支付">
                <ClockCircleTwoTone />
              </Tooltip>
            )}

            {/* icon */}
            {record.statusCode === 1 && record.replenishment && (
              <Tooltip title="补单成功">
                <PushpinTwoTone />
              </Tooltip>
            )}
          </span>
        );
      },
      valueType: 'select',
      valueEnum: TRANSFER_STATUS_MAP,
    },
    {
      title: '商户',
      order: 10,
      dataIndex: 'mchId',
      hideInTable: true,
      valueType: 'select',
      request: requestMerchantOptions,
      fieldProps: {
        showSearch: true,
      },
    },
    {
      title: '大通道',
      order: 9,
      dataIndex: 'productId',
      hideInTable: true,
      valueType: 'select',
      request: requestProductOptions,
      fieldProps: {
        showSearch: true,
      },
    },
    {
      title: '子通道',
      order: 9,
      dataIndex: 'channelId',
      hideInTable: true,
      valueType: 'select',
      request: requestChannelOptions,
      fieldProps: {
        showSearch: true,
      },
    },
    {
      title: '是否补单',
      dataIndex: 'replenishment',
      hideInTable: true,
      valueType: 'select',
      valueEnum: {
        true: {
          text: '是',
        },
        false: {
          text: '否',
        },
      },
    },
    {
      title: '是否超时订单',
      dataIndex: 'overtime',
      hideInTable: true,
      valueType: 'select',
      valueEnum: {
        true: {
          text: '是',
        },
        false: {
          text: '否',
        },
      },
    },
    {
      title: '是否审计',
      dataIndex: 'audit',
      hideInTable: true,
      valueType: 'select',
      valueEnum: {
        true: {
          text: '是',
        },
        false: {
          text: '否',
        },
      },
    },
    {
      title: '创建时间',
      dataIndex: 'createDateRange',
      valueType: 'dateRange',
      hideInTable: true,
      fieldProps: { ranges: DEFAULT_DATE_RANGES },
      search: {
        transform: (value) => {
          return {
            createStartTime: value[0],
            createEndTime: value[1],
          };
        },
      },
    },
    {
      title: '打款时间',
      dataIndex: 'payTimeRange',
      valueType: 'dateRange',
      hideInTable: true,
      fieldProps: { ranges: DEFAULT_DATE_RANGES },
      search: {
        transform: (value) => {
          return {
            payStartTime: value[0],
            payEndTime: value[1],
          };
        },
      },
    },
    {
      title: '完成时间',
      dataIndex: 'finishDateRange',
      valueType: 'dateRange',
      hideInTable: true,
      fieldProps: { ranges: DEFAULT_DATE_RANGES },
      search: {
        transform: (value) => {
          return {
            finishStartTime: value[0],
            finishEndTime: value[1],
          };
        },
      },
    },

    {
      title: '实时订单',
      hideInTable: true,
      dataIndex: 'timeType',
      valueType: 'select',
      valueEnum: timeTypeEnum,
    },
    {
      title: '创建时间',
      width: 135,
      search: false,
      dataIndex: 'createTime',
    },
    {
      title: '打款时间',
      width: 135,
      search: false,
      dataIndex: 'payTime',
    },
    {
      title: '完成时间',
      width: 135,
      search: false,
      dataIndex: 'finishTime',
    },
  ];

  return BASE_COLUMNS;
};
