import { PRODUCT_ICON_ENUM, PROTABLE_DEFAULT_SETTING } from '@/constants';
import ProTable from '@ant-design/pro-table';
import type { ActionType } from '@ant-design/pro-table';
import type { ProColumns } from '@ant-design/pro-table';
import { Avatar, message, Modal, Popconfirm, Progress, Space, Switch, Tag } from 'antd';
import { useRef, useReducer } from 'react';
import ChannelSubEdit from './ChannelSubEdit';
import {
  deletePaymentChannel,
  requestPaymentChannelList,
  switchPaymentChannelStatus,
} from '@/services/product/payment';
import ModalFormCopy from './ModalFormCopy';
import type { ICountReducer } from '@/reducers/modalReducer';
import modalReducer from '@/reducers/modalReducer';
import { PAYMENT_TYPE_ENUM } from '../../constants';
import styles from '../style.less';

const copyData = (record: API.PaymentChannelItem) => {
  const d = { ...record };
  d.channelName = d.channelName ? `${d.channelName}-cp` : '';
  d.channelCode = d.channelCode ? `${d.channelCode}-cp` : '';
  return d;
};

const PRODUCT_TYPE = {
  H5: <Tag color="success">H5</Tag>,
  SDK: <Tag color="processing">SDK</Tag>,
};

function ChannelSub() {
  const actionRef = useRef<ActionType>();
  const [state, dispatch] = useReducer<ICountReducer>(modalReducer, {
    visible: false,
    initialData: {},
  });

  const handleDelete = async (id: string) => {
    const res = await deletePaymentChannel(id);
    if (res?.code === '0000') {
      actionRef.current?.reload();
      message.success('删除成功');
    }
  };

  // 切换状态
  const switchStatus = async (id: string, status: number) => {
    const text = status === 0 ? '禁用' : '启用';
    const res = await switchPaymentChannelStatus(id, status);
    if (res?.code === '0000') {
      actionRef.current?.reload?.();
      message.success(`${text}成功`);
    }
  };

  const showEdit = (record: API.PaymentChannelItem) => {
    dispatch({
      type: 'showModal',
      payload: {
        initialData: record,
      },
    });
  };

  const columns: ProColumns<API.PaymentChannelItem>[] = [
    {
      title: '通道ID',
      dataIndex: 'id',
      width: 65,
    },
    {
      title: '分组名称',
      width: 100,
      dataIndex: 'groupName',
    },
    {
      title: '通道名称',
      width: 200,
      ellipsis: true,
      dataIndex: 'channelName',
      render: (payMethod, record) => (
        <>
          <Avatar src={PRODUCT_ICON_ENUM[record.payMethod]} shape="square" size="small" />
          <span style={{ marginLeft: 8 }}>{record.channelName}</span>
        </>
      ),
    },
    {
      title: '调用代码',
      width: 115,
      search: false,
      dataIndex: 'channelWord',
    },
    {
      title: '通道编码',
      width: 160,
      copyable: true,
      ellipsis: true,
      dataIndex: 'channelCode',
    },
    {
      title: '通道类型',
      width: 80,
      search: false,
      dataIndex: 'channelType',
      render: (channelType, record) => PRODUCT_TYPE[record.channelType || ''],
    },
    {
      title: '币种',
      width: 60,
      search: false,
      dataIndex: 'currency',
    },
    {
      title: '日总限额',
      search: false,
      width: 145,
      dataIndex: 'dayLimitAmount',
      render: (_) => {
        return (
          <Space>
            {_}
            {/* <ModalFormDayLimitAmount initialData={record} refresh={actionRef.current?.reload} /> */}
          </Space>
        );
      },
    },
    {
      title: '今日成功率',
      search: false,
      width: 120,
      dataIndex: 'todaySuccessRate',
      render: (_, record) => (
        <Progress percent={record.todaySuccessRate} steps={10} size="small" strokeColor="#52c41a" />
      ),
    },
    {
      title: '今日成功总计',
      search: false,
      width: 145,
      dataIndex: 'todayTotalAmount',
      render: (_) => <strong className="orange">{_}</strong>,
    },
    {
      title: '今日已使用',
      search: false,
      width: 120,
      dataIndex: 'todayRate',
      render: (_, record) => (
        <Progress percent={record.todayRate} steps={10} size="small" strokeColor="#52c41a" />
      ),
    },
    {
      title: '单笔限额',
      width: 170,
      search: false,
      ellipsis: true,
      dataIndex: 'amountLimit',
    },
    {
      title: '费率',
      dataIndex: 'feeConfig',
      width: 150,
      ellipsis: true,
    },
    {
      title: '状态',
      dataIndex: 'status',
      width: 80,
      render: (_, record) => {
        const { status } = record;
        return (
          <Switch
            // disabled={!googleStatus}
            checkedChildren="启用"
            unCheckedChildren="禁用"
            checked={!!status}
            onClick={switchStatus.bind(null, record.id as string, status === 0 ? 1 : 0)}
          />
        );
      },
      valueType: 'select',
      valueEnum: {
        0: {
          text: '禁用',
        },
        1: {
          text: '启用',
        },
      },
    },
    {
      title: '创建时间',
      search: false,
      width: 135,
      dataIndex: 'createTime',
    },

    {
      title: '操作',
      search: false,
      width: 130,
      fixed: 'right',
      render: (_, record) => (
        <Space>
          <a onClick={showEdit.bind(null, record)}>配置</a>
          <ModalFormCopy
            refresh={actionRef.current?.reset}
            initialData={copyData(record)}
            type={PAYMENT_TYPE_ENUM.CHANNEL}
          />
          <Popconfirm title="确定删除该商户账号？" onConfirm={handleDelete.bind(null, record.id)}>
            <a>删除</a>
          </Popconfirm>
        </Space>
      ),
    },
  ];

  return (
    <>
      <ProTable
        {...PROTABLE_DEFAULT_SETTING}
        scroll={{ x: 1690, y: 'calc(100vh - 340px)' }}
        columns={columns}
        request={requestPaymentChannelList}
        form={{
          colon: false,
        }}
        actionRef={actionRef}
        search={{
          labelWidth: 'auto',
        }}
        defaultSize="small"
      />
      <Modal
        width={790}
        className={styles.subChannelEditModal}
        visible={state.visible}
        onCancel={() => {
          dispatch({ type: 'closeModal' });
        }}
        title={`通道配置：${state.initialData?.channelName}`}
        destroyOnClose
        maskClosable={false}
        footer={null}
      >
        <ChannelSubEdit
          data={state.initialData as API.MerchantChannelItem}
          onCancel={() => {
            dispatch({ type: 'closeModal' });
          }}
          refresh={actionRef.current?.reload}
        />
      </Modal>
    </>
  );
}

export default ChannelSub;
