import { useRef } from 'react';
import { PRODUCT_ICON_ENUM } from '@/constants';
import { editBasic } from '@/services/product/payment';
import ProForm, {
  ProFormDependency,
  ProFormDigit,
  ProFormMoney,
  ProFormRadio,
  ProFormSelect,
  ProFormSwitch,
  ProFormText,
  ProFormTextArea,
} from '@ant-design/pro-form';
import type { ProFormInstance } from '@ant-design/pro-form';
import { Avatar, Button, message, Row, Col, Space } from 'antd';
import { REQUEST_TIMEOUT_OPTIONS } from '../../constants';

interface IProps {
  data: API.MerchantChannelItem;
  onCancel: () => void;
  refresh: () => void;
}

const formItemLayout = {
  labelCol: {
    span: 4,
  },
  wrapperCol: {
    span: 18,
  },
};
const subFormItemLayout = {
  labelCol: {
    span: 8,
  },
  wrapperCol: {
    span: 14,
  },
};
const showFeeFix = (feeType: number) => [2, 3].includes(feeType);
const showFeeRate = (feeType: number) => [1, 3].includes(feeType);

function FormCommon({ data, onCancel, refresh }: IProps) {
  const formRef = useRef<ProFormInstance>();
  const handleFinish = async (values: any) => {
    const params = { id: data.id, ...values, status: values.status ? 1 : 0 };
    const res = await editBasic(params);
    if (res.code === '0000') {
      refresh?.();
      message.success(`编辑成功`);
      return true;
    }
    return false;
  };

  const handleCurrencySelect = (v: any, d: any) => {
    const currencyMark = d.children.split('/')[2];
    formRef.current?.setFieldsValue({
      currencyMark,
    });
  };

  return (
    <div>
      <ProForm
        {...formItemLayout}
        formRef={formRef}
        layout="horizontal"
        initialValues={data}
        onFinish={handleFinish}
        submitter={{
          searchConfig: { submitText: '确认' },
          render: (props, doms) => {
            return (
              <Space style={{ width: '100%', justifyContent: 'flex-end' }}>
                <Button onClick={onCancel}>取消</Button>
                {doms[1]}
              </Space>
            );
          },
        }}
        // onReset={handleReset}
      >
        <Row>
          <Col span={12}>
            <ProFormText
              {...subFormItemLayout}
              name="channelName"
              placeholder="请输入通道名称"
              rules={[{ required: true, message: '请输入通道名称！' }]}
              label="通道名称"
            />
          </Col>
          <Col span={11}>
            <ProFormText
              {...subFormItemLayout}
              placeholder="请输入通道名称"
              name="groupName"
              label="通道分组"
            />
          </Col>
        </Row>
        <Row>
          <Col span={12}>
            <ProFormText
              {...subFormItemLayout}
              name="channelCode"
              placeholder="请输入通道编码"
              rules={[{ required: true, message: '请输入通道编码！' }]}
              label="通道编码"
            />
          </Col>
          <Col span={11}>
            <ProFormText
              {...subFormItemLayout}
              placeholder="请输入调用代码"
              name="channelWord"
              disabled
              label="调用代码"
            />
          </Col>
        </Row>

        <Row>
          <Col span={12}>
            <ProFormRadio.Group
              {...subFormItemLayout}
              options={[
                {
                  label: 'H5',
                  value: 'H5',
                },
                {
                  label: 'SDK',
                  value: 'SDK',
                },
              ]}
              name="channelType"
              label="通道类型"
              placeholder="请输入通道编码"
              rules={[{ required: true, message: '请选择通道编码！' }]}
            />
          </Col>
          <Col span={11}>
            <ProFormSelect
              {...subFormItemLayout}
              name="currency"
              label="货币单位"
              placeholder="请输入货币单位"
              fieldProps={{ onSelect: handleCurrencySelect }}
              valueEnum={{
                CNY: '人民币/CNY/¥',
                INR: '印度卢比/INR/₹',
                USD: '美元/USD/$',
                VND: '越南盾/VND/₫',
                GBP: '英镑/GBP/￡',
                JPY: '日元/JPY/￥',
              }}
              rules={[{ required: true, message: '请选择货币单位！' }]}
            />
          </Col>
        </Row>

        <ProFormRadio.Group
          options={[
            {
              label: (
                <>
                  <Avatar src={PRODUCT_ICON_ENUM.alipay} shape="square" size="small" />
                  &nbsp;支付宝
                </>
              ),
              value: 'alipay',
            },
            {
              label: (
                <>
                  <Avatar src={PRODUCT_ICON_ENUM.wechat} shape="square" size="small" />
                  &nbsp;微信
                </>
              ),
              value: 'wechat',
            },
            {
              label: (
                <>
                  <Avatar src={PRODUCT_ICON_ENUM.unionpay} shape="square" size="small" />
                  &nbsp;银联
                </>
              ),
              value: 'unionpay',
            },
            {
              label: (
                <>
                  <Avatar src={PRODUCT_ICON_ENUM.combine} shape="square" size="small" />
                  &nbsp;聚合
                </>
              ),
              value: 'combine',
            },
          ]}
          name="payMethod"
          label="支付方式"
          rules={[{ required: true, message: '请选择支付方式！' }]}
        />
        <ProFormDependency name={['currencyMark']}>
          {({ currencyMark }) => {
            return (
              <ProFormMoney
                name="dayLimitAmountValue"
                min={0}
                max={20000000}
                label="日总限额"
                placeholder="请输入日总限额"
                customSymbol={currencyMark || '¥'}
                rules={[{ required: true, message: '请输入日总限额!' }]}
              />
            );
          }}
        </ProFormDependency>
        <Row>
          <Col span={12}>
            <ProFormSelect
              {...subFormItemLayout}
              name="requestTimeoutSecond"
              label="请求超时"
              placeholder="请选择请求超时"
              request={async () => REQUEST_TIMEOUT_OPTIONS}
              rules={[{ required: true, message: '请选择请求超时!' }]}
            />
          </Col>
          <Col span={11}>
            <ProFormDigit
              {...subFormItemLayout}
              name="orderExpireMinute"
              min={5}
              max={30}
              fieldProps={{ addonAfter: '分钟' }}
              label="订单过期"
              placeholder="请输入订单过期时间"
              rules={[{ required: true, message: '请输入订单过期时间!' }]}
            />
          </Col>
        </Row>
        <ProFormRadio.Group
          name="feeType"
          label="收费方式"
          options={[
            {
              label: '指定费率',
              value: 1,
            },
            {
              label: '单笔固额',
              value: 2,
            },
            {
              label: '组合收费',
              value: 3,
            },
          ]}
          placeholder="请选择收费方式"
          rules={[{ required: true, message: '请选择收费方式!' }]}
        />
        <ProFormDependency name={['feeType', 'currencyMark']}>
          {({ feeType, currencyMark }) => {
            return (
              <>
                {showFeeRate(feeType) && (
                  <ProFormDigit
                    name="feeRate"
                    min={1}
                    max={100}
                    fieldProps={{ addonAfter: '%' }}
                    label="通道费率"
                    required
                    placeholder="请输入通道费率"
                    rules={[
                      ({ getFieldValue }) => ({
                        validator(_, value) {
                          const feeTypeValue = getFieldValue('feeType');
                          if (!value && showFeeRate(feeTypeValue)) {
                            return Promise.reject('请输入通道费率!');
                          }
                          return Promise.resolve();
                        },
                      }),
                    ]}
                  />
                )}
                {showFeeFix(feeType) && (
                  <ProFormMoney
                    name="feeFix"
                    min={0}
                    max={20000000}
                    label="单笔费用"
                    required
                    customSymbol={currencyMark || '¥'}
                    placeholder="请输入单笔费用"
                    rules={[
                      ({ getFieldValue }) => ({
                        validator(_, value) {
                          const feeTypeValue = getFieldValue('feeType');
                          if (!value && showFeeFix(feeTypeValue)) {
                            return Promise.reject('请输入单笔费用!');
                          }
                          return Promise.resolve();
                        },
                      }),
                    ]}
                  />
                )}
              </>
            );
          }}
        </ProFormDependency>
        <Row>
          <Col span={12}>
            <ProFormSwitch
              {...subFormItemLayout}
              checkedChildren="启用"
              unCheckedChildren="禁用"
              name="cashierDesk"
              label="收银跳转"
              required
            />
          </Col>
          <Col span={11}>
            <ProFormSwitch
              {...subFormItemLayout}
              checkedChildren="启用"
              unCheckedChildren="禁用"
              name="status"
              label="通道状态"
              required
            />
          </Col>
        </Row>

        <ProFormTextArea name="memo" label="通道备注" placeholder="请输入通道备注" />
      </ProForm>
    </div>
  );
}

export default FormCommon;
