import InputArray, { inputArrayValidator } from '@/components/InputArray';
import RangeInput, { rangeInputValidator } from '@/components/RangeInput';
import { editInvokeParam } from '@/services/product/transfer';
import ProForm, {
  ProFormDependency,
  ProFormRadio,
  ProFormSelect,
  ProFormText,
} from '@ant-design/pro-form';
import { Button, message, Row, Col, Space } from 'antd';

interface IProps {
  data: API.MerchantChannelItem;
  onCancel: () => void;
  refresh: () => void;
}

const formItemLayout = {
  labelCol: {
    span: 4,
  },
  wrapperCol: {
    span: 18,
  },
};
const subFormItemLayout = {
  labelCol: {
    span: 8,
  },
  wrapperCol: {
    span: 16,
  },
};

function FormParams({ data, onCancel, refresh }: IProps) {
  const handleFinish = async (values: any) => {
    console.log('values: ', values);

    const params = {
      id: data.id,
      ...values,
    };
    if (values.amountLimitType === 1) {
      params.amountLimitLow = values.amountLimits[0];
      params.amountLimitHigh = values.amountLimits[1];
    }
    console.log(params, 'params');
    try {
      const res = await editInvokeParam(params);
      if (res.code === '0000') {
        refresh?.();
        message.success(`编辑成功`);
        return true;
      }
      return false;
    } catch (err) {
      console.log('err: ', err);
      return false;
    }
  };

  return (
    <div>
      <ProForm
        {...formItemLayout}
        layout="horizontal"
        initialValues={{
          ...data,
          amountLimits: [data.amountLimitLow, data.amountLimitHigh],
        }}
        onFinish={handleFinish}
        submitter={{
          searchConfig: { submitText: '确认' },
          render: (props, doms) => {
            return (
              <Space style={{ width: '100%', justifyContent: 'flex-end' }}>
                <Button onClick={onCancel}>取消</Button>
                {doms[1]}
              </Space>
            );
          },
        }}
      >
        <ProFormText
          name="payGateway"
          label="支付网关"
          placeholder="请输入支付网关"
          rules={[{ required: true, message: '请输入支付网关！' }]}
        />

        <Row>
          <Col span={12}>
            <ProFormSelect
              {...subFormItemLayout}
              name="channelHttpMethod"
              label="请求方式"
              valueEnum={{
                'POST/JSON': 'POST/JSON',
                'POST/FORM': 'POST/FORM',
                GET: 'GET',
              }}
              placeholder="请选择请求方式"
              rules={[{ required: true, message: '请选择请求方式！' }]}
            />
          </Col>
          <Col span={12}>
            <ProFormRadio.Group
              {...subFormItemLayout}
              name="amountScale"
              label="金额单位"
              options={[
                {
                  label: '元',
                  value: 100,
                },
                {
                  label: '角',
                  value: 10,
                },
                {
                  label: '分',
                  value: 1,
                },
              ]}
              placeholder="请选择金额单位"
              rules={[{ required: true, message: '请选择金额单位！' }]}
            />
          </Col>
        </Row>

        <ProFormText
          name="channelMerchantNo"
          label="商户ID"
          placeholder="请输入商户ID"
          rules={[{ required: true, message: '请输入商户ID！' }]}
        />
        <ProFormText
          name="channelAppKey"
          label="应用ID"
          placeholder="请输入应用ID"
          rules={[{ required: true, message: '请输入应用ID！' }]}
        />
        <ProFormText
          name="channelAppSecret"
          label="接口秘钥"
          placeholder="请输入接口秘钥"
          rules={[{ required: true, message: '请输入接口秘钥！' }]}
        />
        <ProFormText
          name="channelSignAlgorithm"
          label="签名算法"
          placeholder="请输入签名算法"
          rules={[{ required: true, message: '请输入签名算法！' }]}
        />
        <ProFormRadio.Group
          name="amountLimitType"
          label="限额类型"
          options={[
            {
              label: '区间金额',
              value: 1,
            },
            {
              label: '固定金额',
              value: 2,
            },
          ]}
          placeholder="请选择限额类型"
          rules={[{ required: true, message: '请选择限额类型！' }]}
        />
        <ProFormDependency name={['amountLimitType', 'currencyMark']}>
          {({ amountLimitType, currencyMark }) => {
            if (amountLimitType === 1) {
              return (
                <ProForm.Item
                  name="amountLimits"
                  label="单笔限额"
                  rules={[
                    {
                      required: true,
                      validator: rangeInputValidator,
                    },
                  ]}
                >
                  <RangeInput prefix={currencyMark} />
                </ProForm.Item>
              );
            } else {
              return (
                <ProForm.Item
                  name="amountLimitRangeArr"
                  label="单笔限额"
                  rules={[
                    {
                      required: true,
                      validator: inputArrayValidator,
                    },
                  ]}
                >
                  <InputArray label="金额" type="money" customSymbol={currencyMark} />
                </ProForm.Item>
              );
            }
          }}
        </ProFormDependency>
        <ProFormText name="extension" placeholder="请输入扩展参数" label="扩展参数" />
      </ProForm>
    </div>
  );
}

export default FormParams;
