/** Request 网络请求工具 更详细的 api 文档: https://github.com/umijs/umi-request */
import { extend } from 'umi-request';
import { message } from 'antd';
import { history } from 'umi';
// import { stringify } from 'querystring';

const codeMessage = {
  200: '服务器成功返回请求的数据。',
  201: '新建或修改数据成功。',
  202: '一个请求已经进入后台排队（异步任务）。',
  204: '删除数据成功。',
  400: '发出的请求有错误，服务器没有进行新建或修改数据的操作。',
  401: '用户没有权限（令牌、用户名、密码错误）。',
  403: '用户得到授权，但是访问是被禁止的。',
  404: '发出的请求针对的是不存在的记录，服务器没有进行操作。',
  406: '请求的格式不可得。',
  410: '请求的资源被永久删除，且不会再得到的。',
  422: '当创建一个对象时，发生一个验证错误。',
  500: '服务器发生错误，请检查服务器。',
  502: '网关错误。',
  503: '服务不可用，服务器暂时过载或维护。',
  504: '网关超时。',
};

/** 异常处理程序 */
const errorHandler = (error: any) => {
  const { response } = error;
  console.log(error, 'response error');
  if (response && response.status) {
    const errorText = codeMessage[response.status];
    message.error(errorText);
  } else if (!response) {
    message.error('无法连接服务器，请检查网络或稍后重试！');
  }
  return response;
};

/** 配置request请求时的默认参数 */
const request = extend({
  // prefix: process.env.REACT_APP_BASEURL,
  errorHandler, // 默认错误处理
  credentials: 'include', // 默认请求是否带上cookie
});

/** 配置SaaS体系 登录 / 续登 / 登出 / 常规异常Message */
request.interceptors.response.use(async (response, options) => {
  // const { url } = response;
  const httpdata = await response.clone().json();
  const { code, msg } = httpdata;

  // if (process.env.NODE_ENV === 'development') {
  //   console.log(response, options);
  // }

  /**
   * 200      服务正常运行返回
   * 12002    Logout Message
   */
  switch (code) {
    case '0000':
      break;
    case '102':
      history.replace('/admin/login');
      break;
    default:
      if (!options.ignoreNotification && code) {
        message.error(msg || '网络走神了~');
      }
      break;
  }

  return response;
});

const get = (url: string, params?: any) => {
  return request<API.RequestResult>(url, {
    method: 'GET',
    params,
  });
};

const post = (url: string, data?: any) => {
  return request<API.RequestResult>(url, {
    method: 'POST',
    data,
  });
};

// 通用的 proTable request function
const commonRequest = async (fetchUrl: string, params: API.PageParams) => {
  const { pageSize, current, ...rest } = params;
  const requestParams = {
    size: pageSize,
    page: current,
    ...rest,
  };

  const res = await get(fetchUrl, requestParams);
  if (res.code === '0000') {
    const { records = [], total = 0 } = res.data || {};
    return {
      data: records,
      success: true,
      extra: res.extend,
      total,
    };
  }
  return {
    data: [],
    success: true,
    total: 0,
  };
};

// 通用的 proTable request function-----不分页
const commonRequestNoPagination = async (fetchUrl: string, params: API.PageParams) => {
  const res = await get(fetchUrl, params);
  if (res?.code === '0000') {
    return {
      data: res.data,
      extra: res.extend,
      success: true,
    };
  }
  return {
    data: [],
    success: true,
    total: 0,
  };
};

export { post, get, commonRequest, commonRequestNoPagination };

export default request;
